from uuid import uuid4
from decimal import Decimal
import boto3
import botocore
from reliabilly.components.data import DataClient, get_query
from reliabilly.components.logger import Logger, log_exception
from reliabilly.components.monitoring import monitor, MonitoringProvider
from reliabilly.settings import Constants, Settings


def get_default_dynamo_resource(client=boto3.resource):  # pragma: no cover
    endpoint_url = None
    # Prevent unit tests from stalling in Windows environments
    if not Settings.USE_DYNAMO:
        return None
    config = botocore.client.Config(
        connect_timeout=Constants.DEFAULT_AWS_CONNECT_TIMEOUT,
        retries={Constants.AWS_MAX_RETRY_ATTEMPTS: Constants.DEFAULT_AWS_MAX_RETRY_COUNT})
    if not Settings.ENV:
        endpoint_url = f'{Settings.DYNAMO_URL}:{Settings.DYNAMO_PORT}'
    return client(Constants.DYNAMO_RESOURCE_KEY, region_name=Settings.AWS_REGION,
                  endpoint_url=endpoint_url, config=config)


def get_default_dynamo_client():
    return get_default_dynamo_resource(boto3.client)


# noinspection PyBroadException
class DynamoDataClient(DataClient):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.logger = Logger()
        self.table_name = kwargs.get(Constants.TABLE, Settings.SERVICE_NAME)
        self.client = kwargs.get(Constants.CLIENT, get_default_dynamo_client())
        self.resource = kwargs.get(Constants.RESOURCE, get_default_dynamo_resource())
        self._initialize_tables()

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_create'))
    def create(self, model):
        model = self.get_converted_model(model)
        result = self.resource.Table(self.table_name).put_item(Item=model)
        return self.check_status(result)

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_update'))
    def update(self, key, model):
        model.id = key
        return self.create(model)

    def get_converted_model(self, model, nest=False):
        empty_keys = list()
        if not isinstance(model, dict):
            model = model.__dict__
        for key, value in model.items():
            if isinstance(value, float):
                model[key] = Decimal(str(value))
                continue
            if not value:
                if key not in empty_keys:
                    empty_keys.append(key)
                continue
            if isinstance(value, dict):
                model[key] = self.get_converted_model(value, True)
                if not model[key]:
                    if key not in empty_keys:
                        empty_keys.append(key)
        for key in empty_keys:
            model.pop(key)
        if not nest:
            if Constants.ID not in model.keys():
                model[Constants.ID] = str(uuid4())
        return model

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_get'))
    def get(self, key):
        self.logger.debug(f'Get item on dynamo for id: {key}')
        self.create_table(self.table_name)
        return self.resource.Table(self.table_name).get_item(Key={Constants.ID: key}).get(
            Constants.DYNAMO_ITEM_KEY, None)

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_get_specific'))
    def get_specific(self, id_value, table_name):
        self.logger.debug(f'Get specific item on dynamo for id: {id_value} from {table_name}')
        self.create_table(self.table_name)
        return self.resource.Table(self.table_name).get_item(
            Key={Constants.ID: id_value}).get(Constants.DYNAMO_ITEM_KEY, None)

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_find'))
    def find(self, key, value):
        self.logger.debug(f'Find called on dynamo key: {key} val: {value}')
        all_items = self.get_all()
        return self.find_item_in_list(all_items, **{key: value})

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_get_all'))
    def get_all(self):
        self.logger.debug(f'Get all called on dynamo')
        self.create_table(self.table_name)
        return self.resource.Table(self.table_name).scan().get(Constants.DYNAMO_ITEMS_KEY, [])

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_count'))
    def count(self):
        return self.resource.Table(self.table_name).scan().get(Constants.DYNAMO_COUNT_KEY, 0)

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_purge'))
    def purge(self):
        self.logger.debug('Purge called on dynamo...')
        success = True
        items = self.get_all()
        for item in items:
            success = success and self.delete(item.get(Constants.ID))
        return success

    @log_exception
    @monitor(monitor_name=MonitoringProvider.get_monitor_prefix(Settings.SERVICE_NAME, 'dynamo_delete'))
    def delete(self, key):
        self.logger.debug(f'Delete on dynamo called id: {key}')
        result = self.resource.Table(self.table_name).delete_item(Key={Constants.ID: key})
        return self.check_status(result)

    @staticmethod
    def check_status(result):
        return result.get(Constants.DYNAMO_RESPONSE_METADATA, {}).get(
            Constants.DYNAMO_SUCCESS_STATUS, Constants.BAD_REQUEST_CODE) == Constants.SUCCESS_RESPONSE_CODE

    @log_exception
    def table_exists(self, table_name):
        try:
            return self.resource.Table(table_name).table_status == 'ACTIVE'
        except Exception:  # pylint: disable=broad-except
            return False

    @log_exception
    def delete_table(self, table_name):
        try:
            return self.check_status(self.client.delete_table(TableName=table_name))
        except Exception:  # pylint: disable=broad-except
            return False

    @log_exception
    def filter(self, **kwargs):
        if not kwargs:
            items = self.get_all()
            return items, 0, len(items), len(items)
        offset, limit, query = get_query(**kwargs)
        all_items = self.get_all()
        return self.find_items_in_list(all_items, offset, limit, **query)

    @log_exception
    def delete_db(self):
        self.logger.debug('Drop db on dynamo called.')
        return self.delete_table(self.table_name)

    def create_table(self, table_name):
        try:
            if not self.table_exists(table_name):
                self.resource.create_table(AttributeDefinitions=[Constants.DYNAMO_ID_TABLE_ATTR],
                                           KeySchema=[Constants.DYNAMO_ID_TABLE_KEY],
                                           ProvisionedThroughput=Constants.DEFAULT_DYNAMO_THROUGHPUT,
                                           TableName=table_name)
            return True
        except Exception as ex:  # pylint: disable=broad-except
            self.logger.info(f'Error creating dynamo table. {ex}', exc_info=True)
            return False

    @staticmethod
    def _get_dictionary_set(items_dictionary):
        converted_list = list()
        for key, value in items_dictionary.items():
            converted_list.append(f'{key}|{value}')
        return set(converted_list)

    def find_item_in_list(self, all_items, **kwargs):
        args_set = self._get_dictionary_set(kwargs)
        match_list = list()
        for item in all_items:
            item_set = self._get_dictionary_set(item)
            if len(item_set.intersection(args_set)) == len(args_set):
                match_list.append(item)
        return match_list

    def find_items_in_list(self, all_items, offset, limit, **kwargs):
        self.logger.info(f'Find called on dynamo with {offset}, {limit}')
        match_list = self.find_item_in_list(all_items, **kwargs)
        return match_list, offset, limit, len(match_list)

    def _tag_table(self, table_arn):
        try:
            self.client.tag_resource(ResourceArn=table_arn, Tags=Constants.DYNAMO_TAGS)
        except Exception:  # pylint: disable=broad-except
            pass

    def _initialize_tables(self):
        if self.create_table(self.table_name):
            self._tag_table(self.resource.Table(self.table_name).table_arn)

    def get_status(self):
        return self.table_exists(self.table_name)



from datetime import timedelta, datetime
from unittest import TestCase
from reliabilly.components.tools.date_utils import get_last_hour_epoch, get_current_epoch


class TestDateUtils(TestCase):

    def test_get_last_hour_epoch(self):
        response = get_last_hour_epoch()
        now = int((datetime.now() - timedelta(hours=1)).timestamp() * 1000)
        self.assertEqual(now, response)

    def test_get_current_epoch_fail(self):
        response = get_current_epoch()
        now = int((datetime.now() + timedelta(seconds=2)).timestamp() * 1000)
        self.assertNotEqual(now, response)

    def test_get_last_hour_epoch_fail(self):
        now = get_last_hour_epoch()
        response = int((datetime.now() - timedelta(seconds=2)).timestamp() * 1000)
        self.assertNotEqual(now, response)



from enum import Enum
from reliabilly.settings import Constants


class ResultTypes(Enum):
    UNKNOWN = 'unknown'
    SUCCESS = 'success'
    ERROR = 'error'
    NO_RESULT = 'no_result'
    THRESHOLD = 'threshold'


class QueryResult:  # pylint: disable=too-few-public-methods
    value: float
    result: ResultTypes
    message: str

    def __init__(self, value=None, result=ResultTypes.UNKNOWN, message=Constants.EMPTY):
        self.value = value
        self.result = result
        self.message = message

    def __str__(self):
        return f'Result Value: {self.value}, Result: {self.result}, Message: {self.message}'



import warnings
from unittest import TestCase
from unittest.mock import MagicMock

from reliabilly.components.data.dynamo_client import DynamoDataClient
from reliabilly.components.models.base import BaseModel
from reliabilly.settings import Constants


TEST_KEY = 'test_key'
TEST_VALUE = 'test_value'
SAMPLE_ITEMS = [{TEST_KEY: TEST_VALUE}]
SAMPLE_ITEMS_RESPONSE = {Constants.DYNAMO_ITEMS_KEY: SAMPLE_ITEMS, Constants.DYNAMO_COUNT_KEY: len(SAMPLE_ITEMS)}
SAMPLE_RESPONSE_SUCCESS = {
    Constants.DYNAMO_RESPONSE_METADATA: {Constants.DYNAMO_SUCCESS_STATUS: Constants.SUCCESS_RESPONSE_CODE}}
SAMPLE_RESPONSE_FAIL = {
    Constants.DYNAMO_RESPONSE_METADATA: {Constants.DYNAMO_SUCCESS_STATUS: Constants.UNAUTHORIZED_CODE}}


class TestDynamoDataClient(TestCase):
    def setUp(self):
        warnings.filterwarnings("ignore", category=ResourceWarning)

    @staticmethod
    def get_mocked_client(mock):
        mock.Table.return_value = mock
        return DynamoDataClient(factory=mock, table_name=Constants.EMPTY, client=mock, resource=mock)

    def test_instantiation(self):
        mock = MagicMock()
        dynamo = DynamoDataClient(factory=mock, table_name=Constants.EMPTY)
        self.assertIsNotNone(dynamo)

    def test_get(self):
        client_mock = MagicMock()
        dynamo = self.get_mocked_client(client_mock)
        self.assertTrue(dynamo.get(Constants.EMPTY))

    def test_get_specific(self):
        client_mock = MagicMock()
        dynamo = self.get_mocked_client(client_mock)
        self.assertTrue(dynamo.get_specific(Constants.EMPTY, Constants.EMPTY))

    def test_create_true(self):
        client_mock = MagicMock()
        client_mock.put_item.return_value = SAMPLE_RESPONSE_SUCCESS
        dynamo = self.get_mocked_client(client_mock)
        self.assertIsNotNone(dynamo)
        result = dynamo.create(BaseModel())
        self.assertTrue(result)

    def test_get_all(self):
        client_mock = MagicMock()
        dynamo = self.get_mocked_client(client_mock)
        self.assertTrue(dynamo.get_all())

    def test_rename_table(self):
        client_mock = MagicMock()
        self.assertIsNotNone(client_mock)

    def test_create_false(self):
        client_mock = MagicMock()
        self.assertIsNotNone(client_mock)

    def test_delete(self):
        client_mock = MagicMock()
        client_mock.delete_item.return_value = SAMPLE_RESPONSE_SUCCESS
        dynamo = self.get_mocked_client(client_mock)
        self.assertTrue(dynamo.delete(TEST_VALUE))

    def test_delete_table(self):
        client_mock = MagicMock()
        client_mock.delete_table.return_value = SAMPLE_RESPONSE_SUCCESS
        dynamo = self.get_mocked_client(client_mock)
        self.assertTrue(dynamo.delete_table(TEST_VALUE))

    def test_find(self):
        client_mock = MagicMock()
        client_mock.scan.return_value = SAMPLE_ITEMS_RESPONSE
        dynamo = self.get_mocked_client(client_mock)
        self.assertEqual(len(dynamo.find(TEST_KEY, TEST_VALUE)), len(SAMPLE_ITEMS))
        self.assertEqual(dynamo.count(), len(SAMPLE_ITEMS))

    def test_update(self):
        client_mock = MagicMock()
        self.assertIsNotNone(client_mock)

    def test_read(self):
        client_mock = MagicMock()
        self.assertIsNotNone(client_mock)

    def test_delete_db(self):
        client_mock = MagicMock()
        client_mock.delete_table.return_value = SAMPLE_RESPONSE_SUCCESS
        dynamo = self.get_mocked_client(client_mock)
        self.assertTrue(dynamo.delete_db())

    def test_purge(self):
        client_mock = MagicMock()
        client_mock.scan.return_value = SAMPLE_ITEMS_RESPONSE
        client_mock.delete_item.return_value = SAMPLE_RESPONSE_SUCCESS
        dynamo = self.get_mocked_client(client_mock)
        self.assertTrue(dynamo.purge())

    def test_find_items(self):
        mock = MagicMock()
        client = DynamoDataClient(factory=mock, table_name=Constants.EMPTY, client=mock, resource=mock)
        items = [{'id': 111, 'test1': 'value1', 'test2': 'value1', 'test3': 1},
                 {'id': 222, 'test1': 'value2', 'test2': 'value1', 'test3': 2},
                 {'id': 333, 'test1': 'value3', 'test2': 'value1', 'test3': 3},
                 {'id': 444, 'test1': 'value4', 'test2': 'value1', 'test3': 4}]
        result, *_ = client.find_items_in_list(items, 0, 5, **{'test1': 'value1', 'test3': 1})
        self.assertEqual(len(result), 1)
        result, *_ = client.find_items_in_list(items, 0, 5, **{'test2': 'value1', 'test3': 1})
        self.assertEqual(len(result), 1)
        result, *_ = client.find_items_in_list(items, 0, 5, **{'test2': 'value1'})
        self.assertEqual(len(result), 4)
        result, *_ = client.find_items_in_list(items, 0, 5, **{'test2': 'value1', 'test3': '4'})
        self.assertEqual(len(result), 1)
        result, *_ = client.find_items_in_list(items, 0, 5, **{'test2': 'value2'})
        self.assertEqual(len(result), 0)

    def test_filter(self):
        mock = MagicMock()
        client = DynamoDataClient(factory=mock, table_name=Constants.EMPTY, client=mock, resource=mock)
        items = [{'id': 111, 'test1': 'value1', 'test2': 'value1', 'test3': 1},
                 {'id': 222, 'test1': 'value2', 'test2': 'value1', 'test3': 2},
                 {'id': 333, 'test1': 'value3', 'test2': 'value1', 'test3': 3},
                 {'id': 444, 'test1': 'value4', 'test2': 'value1', 'test3': 4}]
        mock.Table.return_value = mock
        mock.scan.return_value = {Constants.DYNAMO_ITEMS_KEY: items}
        result, *_ = client.filter()
        self.assertEqual(len(result), len(items))

    def test_get_converted_model(self):
        mock = MagicMock()
        client = DynamoDataClient(factory=mock, table_name=Constants.EMPTY, client=mock, resource=mock)
        model = {'name': 'name', 'portfolio': '', 'test': 1.0, 'dict': {'test': ''}}
        converted_model = client.get_converted_model(model)
        self.assertTrue('portfolio' not in converted_model.keys())
        self.assertTrue('dict' not in converted_model.keys())
        self.assertTrue('name' in converted_model.keys())
        self.assertTrue('test' in converted_model.keys())

    def test_dependency(self):
        mock = MagicMock()
        data_client = DynamoDataClient(factory=mock, client=mock)
        self.assertEqual(data_client, data_client.get_dependency(mock))



import os
from invoke import task
from reliabilly.components.chat.chat_client import ChatClient
from reliabilly.components.chat.message_builder import MessageBuilder
from reliabilly.components.tools.source_control_client import SourceControlClient
from reliabilly.settings import Constants

CHANNEL_NAME = '#slack-channel'


@task
def populate_pull_info(_):
    """ Add pull request details to environment """
    org = Constants.GITHUB_ORG
    repo = Constants.GITHUB_REPO
    branch = os.environ.get('BRANCH_NAME', None)
    commit = os.environ.get('GIT_COMMIT', None)
    source_control_token = os.environ.get('SOURCE_CLIENT_TOKEN', None)
    if branch is not None and branch == Constants.PROD_BRANCH_NAME and source_control_token is not None:
        source_client = SourceControlClient(source_control_token)
        pull = source_client.get_origin_pr(org, repo, commit)
        if not pull:
            return
        title = str(pull.title).replace(' ', '\\ ')
        name = str(pull.user.name).replace(' ', '\\ ')
        url = str(pull.html_url)
        email = str(pull.user.email)
        merged_name = str(pull.merged_by.name).replace(' ', '\\ ')
        merged_email = str(pull.merged_by.email).replace(' ', '\\ ')
        export = 'export CHANGE_TITLE="{0}" CHANGE_AUTHOR_DISPLAY_NAME="{1}" CHANGE_URL="{2}" ' \
                 'CHANGE_AUTHOR_EMAIL="{3}" CHANGE_MERGED_BY="{4}" CHANGE_MERGED_BY_EMAIL="{5}"'.format(
                     title, name, url, email, merged_name, merged_email)
        print(export)
        return


@task
def send_channel_message(_, channel_name, message):
    """ Send slack message to a channel"""
    ChatClient().send_room_message(channel_name, message)


@task
def send_direct_message(_, email, message):
    """ Send slack message to a user"""
    ChatClient().send_user_message(email, message)


@task
def send_pipeline_message(_, message_type, alert_type):
    """ Send a Slack message with info from Jenkins pipeline"""
    environment = os.environ
    builder = MessageBuilder(message_type, environment)
    email = environment.get('CHANGE_AUTHOR_EMAIL', '')
    message = "Jenkins has an update: "
    attachments = builder.make_attachment()
    if alert_type == 'ALERT_USER':
        ChatClient().send_user_message(email, message, attachments=attachments)
    else:
        ChatClient().send_room_message(CHANNEL_NAME, message, attachments=attachments)



# pylint: skip-file
import json
import logging
import unittest
from unittest.mock import MagicMock
from reliabilly.components.controllers.data_handler import DataHandler
from reliabilly.settings import Constants


class TestDataHandler(unittest.TestCase):
    def setUp(self):
        logging.disable(logging.CRITICAL)

    def test_get_single_with_no_item_returns_RESOURCE_NOT_FOUND(self):
        # Setup
        mock_client = MagicMock()
        mock_id = MagicMock()
        mock_client.get.return_value = None
        data_handler = DataHandler(mock_client)

        # Execute
        result = data_handler.handle_get_request_single(mock_id)

        # Assert
        self.assertEqual(result[0], Constants.RESOURCE_NOT_FOUND_CODE)
        self.assertEqual(result[1], Constants.RESOURCE_NOT_FOUND_MESSAGE)

    def test_get_single_with_item_returns_SUCCESS_RESPONSE_CODE(self):
        # Setup
        mock_id = MagicMock()
        mock_data_client = MagicMock()
        item = [{'name': 'item'}]
        mock_data_client.get.return_value = item
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_get_request_single(mock_id)

        # Assert
        self.assertEqual(result[0], Constants.SUCCESS_RESPONSE_CODE)
        self.assertEqual(result[1], json.dumps(item))

    def test_get_with_item_returns_SUCCESS_RESPONSE_CODE(self):
        # Setup
        mock_request = MagicMock()
        mock_data_client = MagicMock()
        items = [{'name1': 'item1'}, {'name2': 'item2'}]
        mock_data_client.count.return_value = 2
        mock_data_client.filter.return_value = [items, 0, 25, 2]
        data_handler = DataHandler(mock_data_client)

        # Execute
        mock_request.args = dict()
        result = data_handler.handle_get_request(mock_request)
        # Assert
        self.assertEqual(result[0], Constants.SUCCESS_RESPONSE_CODE)
        result_data = json.loads(result[1])
        self.assertEqual(result_data[list(result_data)[0]], items)

    def test_get_with_item_returns_BAD_REQUEST_CODE(self):
        # Setup
        mock_request = MagicMock()
        mock_data_client = MagicMock()
        mock_data_client.read.return_value = ValueError
        data_handler = DataHandler(mock_data_client)

        # Execute
        mock_request.args = dict()
        result = data_handler.handle_get_request(mock_request)

        # Assert
        self.assertEqual(result[0], Constants.BAD_REQUEST_CODE)
        self.assertEqual(result[1], Constants.BAD_REQUEST_MESSAGE)

    def test_get_with_negative_limit_returns_BAD_REQUEST_CODE(self):
        # Setup
        mock_request = MagicMock()
        mock_data_client = MagicMock()
        mock_data_client.filter.return_value = ("mock_items", "mock_offset",
                                                -1, "mock_count")
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_get_request(mock_request)

        # Assert
        self.assertEqual(result[0], Constants.BAD_REQUEST_CODE)
        self.assertEqual(result[1], Constants.BAD_REQUEST_MESSAGE)

    def test_post_with_bad_request_returns_BAD_REQUEST(self):
        # Setup
        mock_data_client = MagicMock()
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_post_request(None)

        # Assert
        self.assertEqual(result[0], Constants.BAD_REQUEST_CODE)
        self.assertEqual(result[1], Constants.BAD_REQUEST_MESSAGE)

    def test_post_with_failure_returns_error(self):
        # Setup
        mock_data_client = MagicMock()
        mock_data_client.create.return_value = False
        payload = {'name': 'item'}
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_post_request(payload)

        # Assert
        self.assertEqual(result, "Failed to create new item: {'name': 'item'}")

    def test_post_with_request_returns_CREATED_SUCCESS(self):
        # Setup
        mock_model = MagicMock()
        mock_data_client = MagicMock()
        payload = {'name': 'item'}
        mock_model.add.return_value = payload
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_post_request(payload)

        # Assert
        self.assertEqual(result[0], Constants.CREATED_SUCCESS_CODE)
        self.assertEqual(result[1], json.dumps(payload))

    def test_count_returns_SUCCESS_RESPONSE(self):
        # Setup
        mock_data_client = MagicMock()
        mock_data_client.count.return_value = 10
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_count_request()

        # Assert
        self.assertEqual(result[0], Constants.SUCCESS_RESPONSE_CODE)
        self.assertEqual(result[1], json.dumps(10))

    def test_purge_returns_SUCCESS_RESPONSE(self):
        # Setup
        mock_data_client = MagicMock()
        purge_message = {'message': 'Purge Successful!'}
        mock_data_client.purge.return_value = purge_message
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_purge_request()

        # Assert
        self.assertEqual(result[0], Constants.SUCCESS_RESPONSE_CODE)
        self.assertEqual(result[1], json.dumps(purge_message))

    def test_delete_returns_SUCCESS_RESPONSE(self):
        # Setup
        mock_data_client = MagicMock()
        mock_data_client.delete.return_value = True
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_delete_request(mock_data_client)

        # Assert
        self.assertEqual(result[0], Constants.SUCCESS_RESPONSE_CODE)
        self.assertEqual(result[1], Constants.DELETE_REQUEST_MESSAGE)

    def test_delete_returns_RESOURCE_NOT_FOUND_CODE(self):
        # Setup
        mock_data_client = MagicMock()
        mock_data_client.delete.return_value = False
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_delete_request(mock_data_client)

        # Assert
        self.assertEqual(result[0], Constants.RESOURCE_NOT_FOUND_CODE)
        self.assertEqual(result[1], Constants.RESOURCE_NOT_FOUND_MESSAGE)

    def test_update_returns_SUCCESS_RESPONSE(self):
        # Setup
        mock_data_client = MagicMock()
        mock_data_client.delete.return_value = True
        mock_data_client.create.return_value = True
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_update_request(
            mock_data_client, mock_data_client)

        # Assert
        self.assertEqual(result[0], Constants.CREATED_SUCCESS_CODE)

    def test_update_returns_RESOURCE_NOT_FOUND_CODE(self):
        # Setup
        mock_data_client = MagicMock()
        mock_data_client.delete.return_value = False
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_update_request(
            mock_data_client, mock_data_client)

        # Assert
        self.assertEqual(result[0], Constants.RESOURCE_NOT_FOUND_CODE)
        self.assertEqual(result[1], Constants.RESOURCE_NOT_FOUND_MESSAGE)

    def test_update_returns_BAD_REQUEST_CODE(self):
        # Setup
        mock_data_client = MagicMock()
        data_handler = DataHandler(mock_data_client)

        # Execute
        result = data_handler.handle_update_request(
            mock_data_client, None)

        # Assert
        self.assertEqual(result[0], Constants.BAD_REQUEST_CODE)
        self.assertEqual(result[1], Constants.BAD_REQUEST_MESSAGE)



from unittest import TestCase
from reliabilly.components.data import get_query
from reliabilly.settings import Constants


class TestDataClient(TestCase):
    def test_arg_processing(self):
        offset, limit, query = get_query(offset=1, limit=4, test='jjb')
        self.assertEqual(offset, 1)
        self.assertEqual(limit, 4)
        self.assertEqual(query, {'test': 'jjb'})
        offset, limit, query = get_query(offset=1, limit=44444)
        self.assertEqual(offset, 1)
        self.assertEqual(limit, Constants.MAX_PAGE_SIZE)



from reliabilly.tests import MagicMock, TestCase, logging, warnings, json, skipUnless
from reliabilly.components.tools.stats_client import StatsLogClient, monitor_collection_status, StatsWebClient
from reliabilly.settings import Constants, Settings
from reliabilly.components.web.http_requestor import HttpRequestor


class StatsLogClientTests(TestCase):
    def setUp(self):
        logging.disable(logging.CRITICAL)
        warnings.filterwarnings(Constants.IGNORE, category=ResourceWarning)

    def test_add_timing(self):
        mock = MagicMock()
        stats_client = StatsLogClient(client=mock, secrets_manager=mock, http_client=mock)
        stats_client.add_timing(Constants.DUMMY, Constants.DUMMY_NUM)
        mock.timing.assert_called_once_with(Constants.DUMMY, Constants.DUMMY_NUM)
        mock.timing.side_effect = Exception
        self.assertFalse(stats_client.add_timing(Constants.DUMMY, Constants.DUMMY_NUM))

    def test_update_increment_with_tags(self):
        mock = MagicMock()
        mock.incr.return_value = True
        stats_client = StatsLogClient(client=mock, secrets_manager=mock, http_client=mock)
        name = 'collection_failure'
        value = 1
        tags = {'env': 'local'}
        result = stats_client.update_increment(name, value, tags)
        mock.incr.assert_called_once_with(name, value, tags=tags)
        self.assertTrue(result)

    def test_update_increment_no_tags(self):
        mock = MagicMock()
        mock.incr.return_value = True
        stats_client = StatsLogClient(client=mock, secrets_manager=mock, http_client=mock)
        name = 'collection_failure'
        value = 1
        result = stats_client.update_increment(name, value)
        mock.incr.assert_called_once_with(name, value)
        self.assertTrue(result)

    def test_update_increment(self):
        mock = MagicMock()
        mock.incr.side_effect = Exception
        stats_client = StatsLogClient(client=mock, secrets_manager=mock, http_client=mock)
        name = 'collection_failure'
        value = 1
        self.assertFalse(stats_client.update_increment(name, value))

    def test_add_gauge_defaults(self):
        mock = MagicMock()
        stats_client = StatsLogClient(client=mock, secrets_manager=mock, http_client=mock)
        name = 'test_stat'
        value = 0
        tags = {'tag1': 'tag1_value', 'tag2': 'tag2_value'}
        result = stats_client.add_gauge(name, value, tags)
        mock.gauge.assert_called_once_with(name, value, delta=False, rate=1, tags=tags)
        self.assertTrue(result)

    def test_add_gauge_no_tags(self):
        mock = MagicMock()
        stats_client = StatsLogClient(client=mock, secrets_manager=mock, http_client=mock)
        name = 'test_stat'
        value = 0
        result = stats_client.add_gauge(name, value)
        mock.gauge.assert_called_once_with(name, value, delta=False, rate=1)
        self.assertTrue(result)

    def test_add_gauge_exception(self):
        mock = MagicMock()
        mock.gauge.side_effect = Exception('test')
        stats_client = StatsLogClient(client=mock, secrets_manager=mock, http_client=mock)
        name = 'test_stat'
        value = 0
        tags = {'tag1': 'tag1_value', 'tag2': 'tag2_value'}
        result = stats_client.add_gauge(name, value, tags)
        self.assertFalse(result)

    def test_add_incr(self):
        @monitor_collection_status('local')
        def test_func_true():
            return True
        result = test_func_true()
        self.assertTrue(result)

        @monitor_collection_status('local')
        def test_func_false():
            return False
        result = test_func_false()
        self.assertFalse(result)


class StatsWebClientTests(TestCase):
    def test_web_payload(self):
        payload = StatsWebClient.get_web_metrics_payload(
            Constants.DUMMY, 55.55, {Constants.PRODUCT: Constants.DUMMY, Constants.DUMMY: Constants.DUMMY_NUM})
        self.assertIsNotNone(payload)
        payload_dictionary = json.loads(payload)
        self.assertEqual(payload_dictionary[Constants.METRIC_NAME_KEY], Constants.DUMMY)
        self.assertEqual(payload_dictionary[Constants.METRIC_VALUE_KEY], 55.55)
        self.assertEqual(payload_dictionary[Constants.METRIC_TAG_KEY_PREFIX + Constants.PRODUCT], Constants.DUMMY)
        self.assertEqual(payload_dictionary[Constants.METRIC_TAG_KEY_PREFIX + Constants.DUMMY], Constants.DUMMY_NUM)

    def test_get_web_api_key(self):
        mock = MagicMock()
        stats_client = StatsWebClient(secrets_manager=mock, http_client=mock)
        self.assertIsNotNone(stats_client.get_api_key())
        self.assertEqual(stats_client.get_api_key(Constants.DUMMY), Constants.DUMMY)

    def test_gauge(self):
        mock = MagicMock()
        stats_client = StatsWebClient(secrets_manager=mock, http_client=mock)
        self.assertTrue(stats_client.add_gauge(Constants.DUMMY, Constants.DUMMY_NUM, {}))
        mock.get_secret.side_effect = Exception
        self.assertFalse(stats_client.add_gauge(Constants.DUMMY, Constants.DUMMY_NUM, {}))

    def test_timing(self):
        mock = MagicMock()
        stats_client = StatsWebClient(secrets_manager=mock, http_client=mock)
        self.assertTrue(stats_client.add_timing(Constants.DUMMY, Constants.DUMMY_NUM, {}))
        mock.get_secret.side_effect = Exception
        self.assertFalse(stats_client.add_timing(Constants.DUMMY, Constants.DUMMY_NUM, {}))

    def test_increment(self):
        mock = MagicMock()
        stats_client = StatsWebClient(secrets_manager=mock, http_client=mock)
        self.assertTrue(stats_client.update_increment(Constants.DUMMY, Constants.DUMMY_NUM, {}))
        mock.get_secret.side_effect = Exception
        self.assertFalse(stats_client.update_increment(Constants.DUMMY, Constants.DUMMY_NUM, {}))

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_real_web_send(self):
        mock = MagicMock()
        mock.get_secret.return_value = ''
        stats_client = StatsWebClient(secrets_manager=mock, http_client=HttpRequestor())
        self.assertTrue(stats_client.add_gauge('odb', 20, {Constants.PRODUCT: 'odb'}))



from unittest import TestCase
from unittest.mock import MagicMock

from reliabilly.components.services.okta import check_session
from reliabilly.settings import Constants


class OktaTests(TestCase):
    def test_session_valid(self):
        mock = MagicMock()
        mock.json.return_value = {Constants.STATUS: Constants.OKTA_ACTIVE_STATUS}
        mock.get.return_value = mock
        self.assertTrue(check_session(Constants.EMPTY, mock, mock))

    def test_session_invalid(self):
        mock = MagicMock()
        mock.json.return_value = {Constants.STATUS: Constants.EMPTY}
        mock.get.return_value = mock
        self.assertFalse(check_session(Constants.EMPTY, mock, mock))

    def test_session_error(self):
        mock = MagicMock()
        mock.json.side_effect = Exception
        mock.get.return_value = mock
        self.assertFalse(check_session(Constants.EMPTY, mock, mock))



import json

from nameko.extensions import DependencyProvider
from reliabilly.settings import Constants


class HealthProvider(DependencyProvider):
    health = Constants.HEALTH_UNKNOWN_MESSAGE

    def get_health(self):
        return {Constants.HEALTH: self.health}


class HealthCheckProvider(DependencyProvider):
    def __init__(self, **kwargs):
        self.dependencies = {}
        if kwargs:
            self.dependencies = kwargs

    def get_dependency_count(self):
        return len(self.dependencies)

    def register_dependency(self, dependency_name, dependency):
        self.dependencies[dependency_name] = dependency

    def check_health(self):
        health = {}
        for dependency_name in self.dependencies:
            health[dependency_name] = self.get_dependency_health(dependency_name)
        return json.dumps(health)

    def get_dependency_health(self, dependency_name):
        if self.dependencies[dependency_name]:
            return self.dependencies[dependency_name].get_health()
        return Constants.UNKNOWN_HEALTH_RESPONSE



from unittest import TestCase
from reliabilly.components.chat.message_builder import MessageBuilder, Payload


class MessageBuilderTests(TestCase):
    def test_get_payload_string(self):
        expected = "{0} has failed linting and testing."
        actual = Payload.get('CODE_QUALITY_WARNING', '_TITLE')
        self.assertEqual(expected, actual)

    def test_get_payload_string_fail(self):
        expected = None
        actual = Payload.get('TIGER', '_STYLE')
        self.assertEqual(expected, actual)

    def test_get_payload_empty(self):
        expected = None
        actual = Payload.get('', '')
        self.assertEqual(expected, actual)

    def test_make_attachment(self):
        message_type = 'CODE_QUALITY_SUCCESS'
        environment = {
            'JOB_NAME': 'reliabilly/platform/PR-257',
            'CHANGE_URL': 'https://github.com/wow/great',
            'CHANGE_AUTHOR_DISPLAY_NAME': 'Nathan Fielder',
            'CHANGE_TITLE': 'broke prod'
        }
        builder = MessageBuilder(message_type, environment)
        expected = [{
            'color': 'good',
            'title': Payload.CODE_QUALITY_SUCCESS_TITLE.format(
                'platform / PR-257'),
            'text': Payload.CODE_QUALITY_SUCCESS_TEXT,
            'actions': [{
                'type': 'button',
                'text': ':github: View pull request on Github',
                'url': 'https://github.com/wow/great'
            }],
            'fields': [{
                'title': 'Author',
                'value': 'Nathan Fielder',
                'short': True
            }, {
                'title': 'Description',
                'value': 'broke prod',
                'short': True
            }]
        }]
        actual = builder.make_attachment()
        self.assertEqual(expected, actual)

    def test_all_attachments(self):
        message_type = 'ALL'
        environment = {
            'JOB_NAME': 'bleh/blah/bloh',
            'CHANGE_URL': 'https://github.dev.reliabilly.com/reliabilly/blob',
            'CHANGE_AUTHOR_DISPLAY_NAME': 'Ali G',
            'CHANGE_TITLE': 'wossup britain',
            'GIT_URL': 'https://github.dev.reliabilly.com/reliabilly/platform.git',
            'GIT_COMMIT': 'abc123',
            'RUN_DISPLAY_URL': 'https://jenkins.com/job',
            'UAT_INT_TEST_TARGET': 'https://uat.example',
            'PROD_INT_TEST_TARGET': 'https://example'
        }
        builder = MessageBuilder(message_type, environment)
        expected = [{
            'title': Payload.ALL_TITLE,
            'text': Payload.ALL_TEXT,
            'actions': [{
                'type': 'button',
                'text': Payload.VIEW_GITHUB_LABEL,
                'url': environment['CHANGE_URL']
            }, {
                'type': 'button',
                'text': Payload.VIEW_LATEST_COMMIT_LABEL,
                'url': f"{environment['GIT_URL'][0:-4]}/commit/"
                       f"{environment['GIT_COMMIT']}"
            }, {
                'type': 'button',
                'text': Payload.VIEW_PIPELINE_LABEL,
                'url': environment['RUN_DISPLAY_URL']
            }, {
                'type': 'button',
                'text': Payload.VIEW_UAT_LABEL,
                'url': environment['UAT_INT_TEST_TARGET']
            }, {
                'type': 'button',
                'text': Payload.VIEW_PROD_LABEL,
                'url': environment['PROD_INT_TEST_TARGET']
            }],
            'fields': [{
                'title': 'Author',
                'value': environment['CHANGE_AUTHOR_DISPLAY_NAME'],
                'short': True
            }, {
                'title': 'Description',
                'value': environment['CHANGE_TITLE'],
                'short': True
            }]
        }]
        actual = builder.make_attachment()
        self.assertEqual(expected, actual)



from circuitbreaker import CircuitBreaker
from requests import RequestException


# pylint: disable=too-few-public-methods
class HttpRequestorBreaker:
    def __init__(self, name, failure_threshold=5, recovery_timeout=60, expected_exception=RequestException):
        self.breaker = CircuitBreaker(failure_threshold, recovery_timeout, expected_exception, name)



from reliabilly.components.logger import Logger
from reliabilly.settings import Settings, Constants
from reliabilly.components.web.http_requestor import HttpRequestor


def check_session(session_token, http_client=HttpRequestor(), logger=Logger()):
    try:
        response = http_client.get(Settings.SESSION_URL.format(session_token=session_token)).json()
        return response.get(Constants.STATUS, Constants.EMPTY) == Constants.OKTA_ACTIVE_STATUS
    except Exception as ex:  # pylint: disable=broad-except
        logger.error(f'Error checking session. ex: {ex}')
        return False



from weakref import WeakKeyDictionary
from time import time
from urllib.parse import urlparse
from nameko.extensions import DependencyProvider
from reliabilly.components.logger import Logger
from reliabilly.settings import Constants, Settings
from reliabilly.components.tools.stats_client import StatsLogClient


def get_elapsed_milliseconds(time_start, time_end):
    return abs(round((time_end - time_start) * 1000, 1))


def monitor(monitor_name=Constants.EMPTY):
    def monitor_wrapper(func):
        def wrapper(*args, **kwargs):
            stats = StatsLogClient()
            time_start = time()
            result = func(*args, **kwargs)
            time_end = time()
            elapsed = get_elapsed_milliseconds(time_end, time_start)
            call_count_name = MonitoringProvider.get_monitor_id(Settings.SERVICE_NAME, func.__name__)
            elapsed_name = MonitoringProvider.get_monitor_id(Settings.SERVICE_NAME, func.__name__, True)
            if monitor_name:
                call_count_name = monitor_name + Constants.MONITOR_CALLS_SUFFIX
                elapsed_name = monitor_name + Constants.MONITOR_ELAPSED_SUFFIX
            # Increment the stats for function call counter
            stats.update_increment(call_count_name, Constants.MONITOR_INCR_DEFAULT)
            # Add the timing for the function call time
            stats.add_timing(elapsed_name, elapsed)
            return result
        return wrapper
    return monitor_wrapper


class MonitoringProvider(DependencyProvider):
    timestamps = WeakKeyDictionary()

    def worker_setup(self, worker_ctx):
        self.timestamps[worker_ctx] = time()

    @staticmethod
    def check_large_page(worker_ctx):
        try:
            request_url = MonitoringProvider.get_request_url(worker_ctx)
            page_size = MonitoringProvider.get_page_size_from_url(request_url)
            return page_size > Constants.DEFAULT_PAGE_SIZE
        except Exception:  # pylint: disable=broad-except
            return False

    @staticmethod
    def get_request_url(worker_ctx):
        try:
            request_url = str(worker_ctx.args).split(Constants.SPACE)[1].replace("'", '')
            return request_url
        except Exception:  # pylint: disable=broad-except
            return Constants.EMPTY

    @staticmethod
    def get_page_size_from_url(request_url):
        url_split = urlparse(request_url)
        if Constants.PAGE_SIZE in url_split.query:
            args = url_split.query.split(Constants.AMPERSAND)
            for arg in args:
                if Constants.PAGE_SIZE in arg:
                    return int(arg.split(Constants.EQUAL)[1])
        return False

    @staticmethod
    def get_monitor_id(service_name, method_name, timing=False, large=False, err=False):
        monitor_prefix = MonitoringProvider.get_monitor_prefix(service_name, method_name, err, large)
        if timing:
            return monitor_prefix + Constants.MONITOR_ELAPSED_SUFFIX
        return monitor_prefix + Constants.MONITOR_CALLS_SUFFIX

    @staticmethod
    def get_monitor_prefix(service_name, method_name, err=False, large=False):
        monitor_prefix = f'{Constants.MONITOR_PREFIX}{service_name}.{method_name}'
        if large:
            monitor_prefix += Constants.MONITOR_LARGE_KEY
        if err:
            monitor_prefix += Constants.MONITOR_ERROR_KEY
        return monitor_prefix

    def worker_result(self, worker_ctx, _=None, exc_info=None):
        large_page = self.check_large_page(worker_ctx)
        stats_client = StatsLogClient()
        service_name = worker_ctx.service_name
        method_name = worker_ctx.entrypoint.method_name
        time_end = time()
        time_start = self.timestamps.pop(worker_ctx)
        elapsed = get_elapsed_milliseconds(time_end, time_start)
        monitor_calls_id = MonitoringProvider.get_monitor_id(service_name, method_name, False, large_page, exc_info)
        monitor_elapsed_id = MonitoringProvider.get_monitor_id(service_name, method_name, True, large_page, exc_info)
        # Increment the stats for function call counter
        stats_client.update_increment(monitor_calls_id, Constants.MONITOR_INCR_DEFAULT)
        # Add the timing for the function call time
        stats_client.add_timing(monitor_elapsed_id, elapsed)

    def log_metric_for_checks(self, worker_ctx, metric_key, metric_value=Constants.EMPTY, logger=Logger(__name__)):
        request_url = self.get_request_url(worker_ctx)
        if 'infrastructure_get' in metric_key:
            logger.debug(f'Metric log for {request_url} - {metric_key}: {metric_value}')



from abc import ABC, abstractmethod
from nameko.extensions import DependencyProvider

from reliabilly.components.health import HealthProvider
from reliabilly.components.logger import log_exception
from reliabilly.settings import Constants


@log_exception
def get_query(**args):
    offset = int(get_request_arg(args, Constants.PAGE_OFFSET, Constants.DEFAULT_PAGE_OFFSET))
    limit = int(get_request_arg(args, Constants.PAGE_SIZE, Constants.DEFAULT_PAGE_SIZE))
    if limit > Constants.MAX_PAGE_SIZE:
        limit = Constants.MAX_PAGE_SIZE
    query = dict()
    for key, value in args.items():
        if key not in (Constants.PAGE_OFFSET, Constants.PAGE_SIZE):
            query[key] = value
    return offset, limit, query


@log_exception
def get_request_arg(args, key, default):
    arg = default
    read_arg = args.get(key, None)
    if read_arg:
        arg = read_arg
    return arg


class DataClient(ABC, HealthProvider):
    """ A data client interface to all supported data storage databases. """
    @abstractmethod
    def __init__(self, **kwargs):
        """ Initializer function with keyword arguments that likely vary based on implementation. """

    @abstractmethod
    def create(self, model):
        """ Create a new record in the data store. """

    @abstractmethod
    def update(self, key, model):
        """ Update a record in the data store. """

    @abstractmethod
    def get(self, key):
        """ Get a record by key from the data store. """

    @abstractmethod
    def get_all(self):
        """ Get all records from the data store. """

    @abstractmethod
    def filter(self, **kwargs):
        """ Get all records that match the filter criteria. """

    @abstractmethod
    def count(self):
        """ Get the count of records in the data store. """

    @abstractmethod
    def purge(self):
        """ Purge all records from the data store. """

    @abstractmethod
    def delete(self, key):
        """ Delete a specific record by key from the data store. """

    @abstractmethod
    def get_status(self):
        """ Get the status of the data store for health and connectivity. """

    def get_dependency(self, _):
        """ Return the dependency for nameko service dependency injection support. """
        return self



import json
from collections import OrderedDict
from reliabilly.components.logger import Logger, log_exception
from reliabilly.settings import Constants, Settings


class DataHandler:

    def __init__(self, data_client):
        self.data_client = data_client
        self.logger = Logger()

    @log_exception
    def handle_get_request_single(self, item_id):
        item = self.data_client.get(item_id)
        if item:
            return Constants.SUCCESS_RESPONSE_CODE, self.to_json(item)
        return Constants.RESOURCE_NOT_FOUND_CODE, Constants.RESOURCE_NOT_FOUND_MESSAGE

    def handle_get_request(self, request=None):
        try:
            result = []
            items, offset, limit, count = self.get_filtered_data(request)
            for item in items:
                result.append(item)
            data = OrderedDict()
            data[Settings.SERVICE_NAME] = result
            data['offset'] = offset
            data['limit'] = limit
            data['total'] = count

            if limit & limit < 0:
                raise ValueError
            return Constants.SUCCESS_RESPONSE_CODE, self.to_json(data)
        # pylint: disable=broad-except
        except(ValueError, Exception) as ex:
            self.logger.error(f'Error in general handle_get_request ex: {ex}', exc_info=True)
            return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE

    def get_filtered_data(self, request):
        if request:
            return self.data_client.filter(**request.args)
        return self.data_client.filter()

    @log_exception
    def handle_post_request(self, payload):
        if payload is None:
            return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE
        if self.data_client.create(payload):
            return Constants.CREATED_SUCCESS_CODE, self.to_json(payload)
        error = "Failed to create new item: {}".format(payload)
        return error

    @log_exception
    def handle_restore_request(self, payload):
        if payload is None:
            return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE
        if self.data_client.restore(payload):
            return Constants.CREATED_SUCCESS_CODE, self.to_json(payload)
        error = "Failed to restore items: {}".format(payload)
        return error

    @log_exception
    def handle_count_request(self):
        return Constants.SUCCESS_RESPONSE_CODE, self.to_json(self.data_client.count())

    @log_exception
    def handle_purge_request(self):
        self.data_client.purge()
        return Constants.SUCCESS_RESPONSE_CODE, Constants.PURGE_REQUEST_MESSAGE

    @log_exception
    def handle_delete_request(self, item_id):
        result = self.data_client.delete(item_id)
        if result:
            return Constants.SUCCESS_RESPONSE_CODE, Constants.DELETE_REQUEST_MESSAGE
        return Constants.RESOURCE_NOT_FOUND_CODE, Constants.RESOURCE_NOT_FOUND_MESSAGE

    @log_exception
    def handle_update_request(self, item_id, payload):
        if payload is None:
            return Constants.BAD_REQUEST_CODE, Constants.BAD_REQUEST_MESSAGE
        if self.data_client.delete(item_id):
            return self.handle_post_request(payload)
        return Constants.RESOURCE_NOT_FOUND_CODE, Constants.RESOURCE_NOT_FOUND_MESSAGE

    @staticmethod
    @log_exception
    def to_json(model):
        return json.dumps(model, default=str)



from nameko.runners import ServiceRunner
from .controllers.data_mongo_controller import DataMongoController
from .settings import Constants

RUNNER = ServiceRunner(config=Constants.NAMEKO_CONFIG)
RUNNER.add_service(DataMongoController)



# flake8: noqa
# pylint: disable=trailing-whitespace
"""
@api get /jjb/
description: Returns all jjb
summary: Fetch all jjb
tags:
    - jjb
parameters:
    - name: limit
      in: query
      description: The maximum number of jjb to fetch
      schema:
        type: integer
    - name: offset
      in: query
      description: The number of jjb to offset your query by
      schema:
        type: integer
responses:
    "200":
        description: A list of jjb items
    "400":
        description: The request was badly formed or otherwise invalid
    "403":
        description: The auth token was invalid or not provided
"""

"""
@api post /jjb/
description: >-
    Submit a new jjb item to the service. Do note that not
    all services necessarily support the addition of new items. If it doesn't,
    it shouldn't have this entry but as the documentation is generated,
    the creator may have forgotten this segment.
summary: Add a new  item to the service
tags:
    - jjb
responses:
    "201":
        description: Successfully submitted the new jjb item
    "400":
        description: The request was badly formed or otherwise invalid
    "403":
        description: The auth token was invalid or not provided
"""

"""
@api get /jjb/{id}
description: Fetch a single jjb item by ID
summary: Fetch a single jjb item by ID
tags:
    - jjb
responses:
    "200":
        description: Successfully retrieved item
    "403":
        description: The auth token was invalid or not provided
    "500":
        description: The ID provided is invalid
"""

"""
@api put /jjb/{id}
description: Update a jjb item in the service
summary: Update a jjb item in the service
tags:
    - jjb
responses:
    "202":
        description: A success message
    "400":
        description: The request was badly formed or otherwise invalid
    "403":
        description: The auth token was invalid or not provided
    "404":
        description: The item was not found. It was deleted or never existed.
"""

"""
@api delete /jjb/{id}
description: Delete a jjb item from the service
summary: Delete a jjb item from the service
tags:
    - jjb
responses:
    "200":
        description: The item was successfully deleted
    "403":
        description: The auth token was invalid or not provided
    "404":
        description: The item was not found. It was deleted or never existed.
"""

"""
@api get /jjb/collect/
description: >-
    Performing a GET on the /collect/ endpoint will show any
    currently running collections that have been manually triggered.
    You can see which collector is running and when it was started.
summary: View currently running collections
tags:
    - jjb
responses:
    "200":
        description: A JSON object containing the status of active collectors
    "403":
        description: The auth token was invalid or not provided
"""

"""
@api post /jjb/collect/
description: >-
    Triggers a manual collection for the jjb service. You
    don't need to do this as services will manually run collections at regular
    intervals but it can be useful if you 100% fresh data.
summary: Fetch the latest jjb data
tags:
    - jjb
responses:
    "202":
        description: A success message stating "Collection received!"
    "403":
        description: The auth token was invalid or not provided
"""

"""
@api post /jjb/purge/
description: Purges all jjb from the service
summary: Purge all data from the jjb service
tags:
    - jjb
responses:
    "200":
        description: The purge was successful
    "403":
        description: The auth token was invalid or not provided
"""

"""
@api get /jjb/ping/
description: Returns a success code if the service is available
summary: Ping the jjb service
tags:
    - jjb
responses:
    "200":
        description: The string "pong" indicating the request succeeded
    "403":
        description: The auth token was invalid or not provided
"""


"""
@api get /jjb/count/
description: See how many jjb are stored in the service
summary: See how many jjb are stored in the service
tags:
    - jjb
responses:
    "200":
        description: An integer reflecting the total number of jjb
    "403":
        description: The auth token was invalid or not provided
"""

"""
@api get /jjb/version/
description: Check the current version of the jjb service
summary: Check the current version of the jjb service
tags:
    - jjb
responses:
    "200":
        description: A JSON object showing version and build number
    "403":
        description: The auth token was invalid or not provided
"""



import json

from reliabilly.components.services.okta import check_session
from reliabilly.settings import Settings, Constants
from reliabilly.components.tools.secret_manager import SecretManager


def get_permitted_no_authentication(request):
    try:
        if request.method == Constants.OPTIONS_REQUEST:
            return True
        route_path_split = list(filter(None, request.path.split(Constants.FORWARD_SLASH)))
        return f'{route_path_split[-1]}{Constants.FORWARD_SLASH}' in \
               [Constants.PING_ROUTE, Constants.VERSION_ROUTE, Constants.HEALTH_ROUTE, Constants.CALENDAR_ROUTE]
    except Exception:  # pylint: disable=broad-except
        return False


def get_acceptable_token_list(version=Settings.VERSION, secret_manager=SecretManager()):
    token_list = [version]
    if Settings.ENV and Settings.ENV != Constants.LOCAL:
        tokens = secret_manager.get_secret(Constants.DEV_TOKEN)
        if tokens:
            token_list.extend(json.loads(tokens))
    return token_list


class Authorization:
    """ DependencyProvider giving services authorization """

    @staticmethod
    def get_auth_token(request):
        if not request or not request.headers:
            return None
        auth_token = request.headers.get(Constants.AUTHORIZATION_HEADER, None)
        return auth_token

    @staticmethod
    def get_auth_token_log(request):
        auth_token = Authorization.get_auth_token(request)
        if auth_token:
            if len(auth_token) > 3:
                return auth_token[-3:]
        return Constants.NONE

    @staticmethod
    def is_permitted(request, token_list_func=get_acceptable_token_list, check_session_fn=check_session):
        if get_permitted_no_authentication(request):
            return True
        auth_token = Authorization.get_auth_token(request)
        if not auth_token:
            return False
        if Constants.BEARER_PREFIX in auth_token:
            return check_session_fn(auth_token)
        return auth_token in token_list_func()

    @staticmethod
    def get_unauthorized_response():
        return Constants.UNAUTHORIZED_CODE, Constants.EMPTY, Constants.UNAUTHORIZED_REQUEST_MESSAGE



from unittest import TestCase
from unittest.mock import MagicMock
from reliabilly.components.logger import LoggingProvider, SumoLogClient, PingLogFilter
from reliabilly.settings import Constants


class LoggingTests(TestCase):
    @staticmethod
    def test_log_debug():
        mock = MagicMock()
        logger = LoggingProvider(Constants.DUMMY, mock, mock)
        logger.debug(Constants.DUMMY)
        mock.debug.assert_called_once()

    @staticmethod
    def test_log_error():
        mock = MagicMock()
        logger = LoggingProvider(Constants.DUMMY, mock, mock)
        logger.error(Constants.DUMMY)
        mock.error.assert_called_once()

    @staticmethod
    def test_log_info():
        mock = MagicMock()
        logger = LoggingProvider(Constants.DUMMY, mock, mock)
        logger.info(Constants.DUMMY)
        mock.info.assert_called_once()

    @staticmethod
    def test_log_critical():
        mock = MagicMock()
        logger = LoggingProvider(Constants.DUMMY, mock, mock)
        logger.critical(Constants.DUMMY)
        mock.critical.assert_called_once()

    def test_get_dependency(self):
        mock = MagicMock()
        logger = LoggingProvider(Constants.DUMMY, mock, mock)
        self.assertEqual(logger.get_dependency(Constants.DUMMY), mock)

    def test_worker_setup(self):
        mock = MagicMock()
        mock.service_name = Constants.DUMMY
        logger = LoggingProvider(Constants.DUMMY, mock, mock)
        logger.worker_setup(mock)
        self.assertEqual(logger.log_name, Constants.DUMMY)

    def test_logger(self):
        logger = LoggingProvider(Constants.DUMMY)
        self.assertIsNotNone(logger)

    def test_ping_filter(self):
        mock = MagicMock()
        mock.getMessage.return_value = 'ping test'
        filter_client = PingLogFilter()
        self.assertFalse(filter_client.filter(mock))
        mock.getMessage.return_value = 'test not'
        self.assertTrue(filter_client.filter(mock))

    def test_sumo_logger(self):
        mock = MagicMock()
        mock.status_code = 200
        mock.post.return_value = mock
        logger = SumoLogClient(mock, 'dummy')
        self.assertEqual(logger.get_environment(), Constants.UAT)
        self.assertEqual(logger.get_environment(Constants.PROD_REGION),
                         Constants.PROD)
        headers = logger.get_headers('jjb-ut')
        self.assertIsNotNone(headers)
        self.assertTrue(logger.ship_log('debug', 'jjb-ut', 'test message'))
        mock.status_code = 404
        self.assertFalse(logger.ship_log('debug', 'jjb-ut', 'test message'))
        mock.post.side_effect = Exception
        self.assertFalse(logger.ship_log('debug', 'jjb-ut', 'test message'))



import unittest
from unittest.mock import MagicMock

from reliabilly.components.chat.slack_client import SlackClient
from reliabilly.settings import Constants

MOCK_RETURN = {'ok': True, 'channel': 'C9Q0CTJBV',
               'ts': '1521062111.000062', 'message':
                   {'text': 'Test',
                    'username': 'Test',
                    'bot_id': 'B5CGL790F', 'type': 'message',
                    'subtype': 'bot_message', 'ts': '1521062111.000062'}}
MOCK_USERS = {'users': [{'slack': {'id': 'hello'}}], 'total': 1}


class SlackClientTests(unittest.TestCase):
    def test_send_message(self):
        mock = MagicMock()
        mock.api_call.return_value = MOCK_RETURN
        slack_client = SlackClient(mock, mock)
        self.assertTrue(slack_client.send_message(Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.api_call.called)

    def test_send_dm(self):
        mock = MagicMock()
        request_mock = MagicMock()
        request_mock.json.return_value = MOCK_USERS
        mock.get.return_value = request_mock
        mock.api_call.return_value = MOCK_RETURN
        slack_client = SlackClient(mock, mock)
        self.assertTrue(slack_client.send_dm(Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.api_call.called)

    def test_update_message(self):
        mock = MagicMock()
        mock.api_call.return_value = MOCK_RETURN
        slack_client = SlackClient(mock, mock)
        self.assertTrue(slack_client.update_message(Constants.DUMMY, Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.api_call.called)

    def test_send_ephemeral_message(self):
        mock = MagicMock()
        mock.api_call.return_value = MOCK_RETURN
        slack_client = SlackClient(mock, mock)
        self.assertTrue(slack_client.send_ephemeral_message(Constants.DUMMY, Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.api_call.called)

    def test_add_reaction(self):
        mock = MagicMock()
        mock.api_call.return_value = MOCK_RETURN
        slack_client = SlackClient(mock, mock)
        self.assertTrue(slack_client.add_reaction(Constants.DUMMY, Constants.DUMMY, Constants.DUMMY))
        self.assertTrue(mock.api_call.called)



from reliabilly.settings import get_service_url
from reliabilly.components.web.http_requestor import HttpRequestor


class PagedRequester(HttpRequestor):
    def __init__(self, service_name, **kwargs):
        super().__init__(**kwargs)
        self.service_name = service_name
        self.current_offset = 0
        self.total = None

    def has_next(self):
        if self.total is not None:
            return self.current_offset < self.total
        return None

    def get_next_page(self):
        results = list()
        headers = self.get_request_headers()
        result = self._call_endpoint(get_service_url(self.service_name), self.current_offset, headers=headers)
        self.current_offset += result['limit']
        if self.total is None:
            self.total = result['total']
        return results

    def get_all(self):
        results = list()
        results += self.get_next_page()
        while self.has_next():
            results += self.get_next_page()
        return results



import warnings
from unittest import TestCase
from reliabilly.components.authorization import Authorization
from reliabilly.components.factories.factory import Factory, Components


class FactoryTests(TestCase):

    def setUp(self):
        warnings.filterwarnings("ignore", category=ResourceWarning)

    def test_dynamo_factory(self):
        client = Factory.create(Components.DataClients.DYNAMODB)
        self.assertIsNotNone(client)

    def test_auth_factory(self):
        client = Factory.create(Components.ServiceClients.AUTH)
        self.assertTrue(isinstance(client, Authorization))



import unittest
from unittest.mock import MagicMock

from reliabilly.components.tools.newrelic_http_client import NewRelicHttpClient


class NewRelicHttpClientTests(unittest.TestCase):

    def test_generate_headers(self):
        client = NewRelicHttpClient("sand-rabbits", "http://nice.com")
        actual = client.generate_headers()
        expected = {'Content-Type': 'application/json', 'X-Insert-Key': 'sand-rabbits'}
        self.assertEqual(expected, actual)

    def test_create_event(self):
        mock = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 202
        mock.post.return_value = mock_response
        payload = {'eventType': 'release', 'title': 'Does not really matter'}
        client = NewRelicHttpClient('huh', 'http://neat.com')
        actual = client.create_event(payload, mock)
        self.assertTrue(actual)

    def test_create_event_failure(self):
        mock = MagicMock()
        mock_response = MagicMock()
        mock_response.status_code = 400
        mock.post.return_value = mock_response
        payload = {'eventType': 'release', 'title': 'Does not really matter'}
        client = NewRelicHttpClient('huh', 'http://neat.com')
        actual = client.create_event(payload, mock)
        self.assertFalse(actual)



from urllib.parse import quote
from reliabilly.components.logger import Logger, log_exception
from reliabilly.components.web.http_requestor import HttpRequestor
from reliabilly.components.services import Constants, ResultTypes, QueryResult
from reliabilly.settings import Settings
from reliabilly.components.tools.secret_manager import SecretManager


@log_exception
def get_newrelic_account_id(account_name):
    return Constants.NEWRELIC_ACCOUNT_IDS.get(account_name, None)


@log_exception
def get_newrelic_api_key(account_name, secrets_manager):
    api_key = None
    api_secrets_name = Constants.NEWRELIC_API_KEYS.get(account_name, None)
    if Settings.LOCAL_NR_API_KEY:
        return Settings.LOCAL_NR_API_KEY
    if api_secrets_name:
        api_key = secrets_manager.get_secret(api_secrets_name)
    return api_key


class NewRelicQueryExecutor:

    def __init__(self, account_name, http=HttpRequestor(), secrets_manager=SecretManager()):
        self.http = http
        self.secrets_manager = secrets_manager
        self.account_name = account_name
        self.account_id = get_newrelic_account_id(self.account_name)
        self.api_key = get_newrelic_api_key(self.account_name, self.secrets_manager)
        self.logger = Logger(Settings.SERVICE_NAME)

    def execute_faceted_query(self, query):
        result = self._execute_query(query)
        result_json = result.json()
        if not result.ok:
            return self._get_failure_result(result_json)
        return QueryResult(result_json, ResultTypes.SUCCESS)

    def execute_single_value_query(self, query):
        try:
            result = self._execute_query(query)
            result_json = result.json()
            if not result.ok:
                return self._get_failure_result(result_json)
            results = result_json.get(Constants.RESULTS, None)
            if results:
                first_result = results[0]
                if first_result:
                    filtered_result = next(iter(first_result.values()))
                    return QueryResult(filtered_result, ResultTypes.SUCCESS)
                return QueryResult(None, ResultTypes.UNKNOWN, Constants.NR_EMPTY_RESULTS_MESSAGE)
            return QueryResult(None, ResultTypes.UNKNOWN, Constants.NR_NO_RESULT_ITEM_MESSAGE)
        except (Exception, ValueError) as ex:  # pylint: disable=broad-except
            self.logger.error(f'Newrelic query execution for {query} failed: {ex}')
            return QueryResult(None, ResultTypes.ERROR, ex)

    def _get_failure_result(self, result_json):
        self.logger.info(f'Newrelic result: {result_json}')
        return QueryResult(None, ResultTypes.ERROR, result_json)

    def build_url(self, account_id, query):
        try:
            url = Settings.NEWRELIC_QUERY_URL
            encoded_query = quote(query)
            return url.format(account_id=account_id, query=encoded_query)
        except TypeError as ex:
            self.logger.error(f'Encoding query failed due to non-string parameter: {ex}')
            return None

    def _execute_query(self, query):
        self.logger.info(f'Executing newrelic query: {query}')
        headers = {Constants.X_QUERY_KEY_HEADER: self.api_key}
        url = self.build_url(self.account_id, query)
        return self.http.get(url, headers=headers)



import boto3
from botocore.exceptions import ClientError
from reliabilly.components.logger import LoggingProvider
from reliabilly.settings import Settings


def get_default_secret_manager():
    return boto3.client('secretsmanager', region_name=Settings.AWS_REGION)


# pylint: disable=too-few-public-methods
class SecretManager:
    def __init__(self, client=get_default_secret_manager(), logger=LoggingProvider(__name__)):
        self.logger = logger
        self.client = client

    def get_secret(self, secret_key):
        try:
            response = self.client.get_secret_value(SecretId=secret_key)
            return response['SecretString']
        except (ClientError, Exception) as ex:  # pylint: disable=broad-except
            self.logger.error(ex)
            return None



from unittest import TestCase
from unittest.mock import MagicMock
from reliabilly.components.web.paged_requester import PagedRequester

DUMMY = "dummy"


class PagedRequesterTests(TestCase):
    def test_init(self):
        mock = MagicMock()
        paged_requester = PagedRequester(DUMMY, client=mock,
                                         circuit_breaker=mock)
        self.assertEqual(paged_requester.current_offset, 0)
        self.assertEqual(paged_requester.total, None)
        self.assertEqual(paged_requester.circuit_breaker, mock)
        self.assertEqual(paged_requester.client, mock)

    def test_has_next_with_none_total(self):
        paged_requester = PagedRequester(DUMMY)
        has_next = paged_requester.has_next()
        self.assertIsNone(has_next)

    def test_has_next_with_total(self):
        mock = MagicMock()
        json_mock = MagicMock()
        json_mock.json.return_value = {"dummy": [], "total": 0, "limit": 25}
        mock.request.return_value = json_mock
        paged_requester = PagedRequester(DUMMY, client=mock)
        paged_requester.get_next_page()
        self.assertIsNotNone(paged_requester.total)
        has_next = paged_requester.has_next()
        self.assertFalse(has_next)

    def test_has_next_true(self):
        mock = MagicMock()
        json_mock = MagicMock()
        json_mock.json.return_value = {"dummy": [{"item": 1, "item2": 2}], "total": 2, "limit": 1}
        mock.request.return_value = json_mock
        paged_requester = PagedRequester(DUMMY, client=mock)
        paged_requester.get_next_page()
        self.assertIsNotNone(paged_requester.total)
        self.assertTrue(paged_requester.has_next())
        self.assertEqual(paged_requester.current_offset, 1)

    def test_has_next_false_at_end(self):
        mock = MagicMock()
        json_mock = MagicMock()
        json_mock.json.return_value = {"dummy": [{"item": 1, "item2": 2}], "total": 2, "limit": 1}
        mock.request.return_value = json_mock
        paged_requester = PagedRequester(DUMMY, client=mock)
        paged_requester.get_next_page()
        paged_requester.get_next_page()
        self.assertIsNotNone(paged_requester.total)
        self.assertFalse(paged_requester.has_next())

    def test_get_all(self):
        mock = MagicMock()
        json_mock = MagicMock()
        json_mock.json.return_value = {"dummy": [{"item": 1, "item2": 2}], "total": 2, "limit": 1}
        mock.request.return_value = json_mock
        paged_requester = PagedRequester(DUMMY, client=mock)
        paged_requester.get_all()
        self.assertEqual(2, mock.request.call_count)

    def test_get_all_with_3(self):
        mock = MagicMock()
        json_mock = MagicMock()
        json_mock.json.return_value = {"dummy": [{"item": 1, "item2": 2}], "total": 3, "limit": 1}
        mock.request.return_value = json_mock
        paged_requester = PagedRequester(DUMMY, client=mock)
        paged_requester.get_all()
        self.assertEqual(3, mock.request.call_count)



from os import path, walk, makedirs
from jinja2 import Environment, FileSystemLoader
from inflection import camelize
from reliabilly.settings import Constants


def camelcase(item):
    return camelize(item)


class Generator:
    def __init__(self, **kwargs):
        self.open = kwargs.get(Constants.OPEN, open)
        self.mkdirs = kwargs.get(Constants.MKDIRS, makedirs)

    def replace_template_file(self, template_path, template_file, destination, **kwargs):
        environment = Environment(autoescape=True, loader=FileSystemLoader(template_path))
        service_name = kwargs.get(Constants.SERVICE_KEY, Constants.EMPTY)
        final_destination = destination.replace(Constants.SERVICE_REPLACE, service_name)[:-3]
        if not any(substring in template_file for substring in Constants.IGNORED_FILES):
            environment.filters[Constants.CAMELCASE_FILTER] = camelcase
            template = environment.get_template(template_file)
            self.write_converted_file(environment, final_destination, kwargs, template)

    def write_converted_file(self, environment, final_destination, kwargs, template):
        with self.open(final_destination, Constants.WRITE_FLAG) as file:
            file.write(template.render(kwargs, environment=environment))

    def process_template_files(self, service_name, priority, destination_dir=Constants.CURRENT_DIR):
        destination = path.join(destination_dir, service_name)
        makedirs(destination, exist_ok=True)
        current_dir = path.dirname(path.realpath(__file__))
        template_dir = path.join(current_dir, Constants.TEMPLATE_DIR)
        for directory, _, file_list in walk(template_dir):
            for file in file_list:
                new_path = destination + path.join(directory, file).split(Constants.TEMPLATE_DIR)[1]
                self. mkdirs(path.dirname(new_path), exist_ok=True)
                self.replace_template_file(directory, file, new_path, service=service_name, priority=priority)
        return True

    def scaffold_up_service(self, service_name, priority):
        return self.process_template_files(service_name, priority)


def scaffold_up_service(service_name, priority, generator=Generator()):
    return generator.scaffold_up_service(service_name, priority)



# pylint: disable=too-few-public-methods
import json
from os import environ, listdir, path


class Constants:
    SPACE = ' '
    NEWLINE = '\n'
    DUMMY = 'dummy'
    WRITE_FLAG = 'w+'
    SERVICE_KEY = 'service'
    SERVICE_REPLACE = '{{service}}'
    TEMPLATE_DIR = 'templates'
    IGNORED_FILES = ['.DS_Store']
    CAMELCASE_FILTER = 'camelcase'
    OPEN = 'open'
    MKDIRS = 'mkdirs'
    HEALTH = 'health'
    HEALTH_UNKNOWN_MESSAGE = 'health unknown'
    UNKNOWN_HEALTH_RESPONSE = {HEALTH: HEALTH_UNKNOWN_MESSAGE}
    MONGO_STATS_COMMAND = 'dbstats'
    MONGO_CONNECTION_HEALTH_ERROR = 'unable to connect to mongodb at {host}:{port}'
    MONGO_HEALTH_ID = 'mongo'
    DYNAMO_HEALTH_ID = 'dynamo'
    COMPOSE_FILE_FLAG_SEPARATOR = ' -f '
    COMPOSE_FILE_NAME = 'docker-compose.yml'
    MONITORING_POLICY_URL = 'https://api.newrelic.com/v2/alerts_synthetics_conditions/policies'
    SERVICE_NAME_ENV = 'SERVICE_NAME'
    MONGO_PORT_ENV = 'MONGO_PORT'
    MONGO_URL_ENV = 'MONGO_URL'
    MONGO_URL_DEFAULT = 'mongodb://mongodb'
    RUN_SKIPPED_ENV = 'RUN_SKIPPED'
    RUN_SKIPPED_MSG = 'skipping due to skip not being set'
    VERSION_KEY = 'version'
    APP_PORT_ENV = 'APP_PORT'
    PORT_ENV_SUFFIX = '_PORT'
    PYTHON_MODULE_FILE = '__init__.py'
    INTEGRATION_WILDCARD = '*.yml'
    DOCKER_FILE = 'Dockerfile'
    CURRENT_DIR = '.'
    TESTS_DIR = 'tests'
    SERVICE_DIR = 'services'
    INTEGRATION_TEST_DIR = 'integration'
    DEFAULT_TARGET = 'http://localhost:8080'
    PRODUCTION_URL_BASE = 'https://reliabilly.com'
    BACKUPS_BUCKET = 's3://reliabilly-backups'
    DUMMY_NUM = 420
    DEFAULT_MONGO_PORT = 27017
    DV_URL_ENV = 'DV_URL'
    DV_URL_DEFAULT = 'https://dvt.reliabilly.com/v1/'
    DEFAULT_APP_PORT = 8080
    STARTING_PORT = 5000
    PORT_INCREMENT = 1
    STARTING_PRIORITY = 100
    DEFAULT_VERSION_TIMEOUT = 120
    MONITORING_URL = 'https://synthetics.newrelic.com/synthetics/api/v3/monitors'
    AUTH_TOKEN = 'AUTH_TOKEN'
    ENVIRONMENT = 'ENVIRONMENT'
    LOCAL_URL_BASE = 'http://localhost:8080/'
    LOCAL_SERVICE_URL_BASE = 'http://{0}:8000/'
    LOCAL_STATIC_URL_BASE = 'http://{0}:80/'
    PROD_URL_BASE = 'https://reliabilly.com/'
    UAT_URL_BASE = 'https://reliabilly-uat.com/'
    ACCEPT_HEADER = 'Accept'
    ACCOUNT_NAME = 'account_name'
    ALL_KEY = 'All'
    AMPERSAND = '&'
    AUTHORIZATION_HEADER = 'Authorization'
    AWS_ACCOUNT = 'AWS_ACCOUNT'
    AWS_REGION = 'AWS_REGION'
    BAD_REQUEST_CODE = 400
    BAD_REQUEST_MESSAGE = json.dumps({'message': 'Invalid request'})
    BEARER_PREFIX = 'Bearer '
    BRANCH_NAME_ENV = 'BRANCH_NAME'
    BUILD_NUMBER_ENV = 'BUILD_NUMBER'
    BUILD_URL_ENV = 'RUN_DISPLAY_URL'
    CALENDAR_ROUTE = 'calendar/'
    CHANNEL_NAME = 'channel_name'
    CIRCUIT_BREAKER = 'circuit_breaker'
    CLIENT = 'client'
    CODE = 'Code'
    COLLECTION_ENABLED = 'COLLECTION_ENABLED'
    COLLECTION_FAILED_MESSAGE = json.dumps({'message': 'Collection Failed!'})
    COLLECTION_FAILED_STATUS = 'Collection failed!'
    COLLECTION_INTERVAL_DEFAULT = 900
    COLLECTION_INTERVAL_SECONDS_KEY = 'COLLECTION_INTERVAL_SECONDS'
    COLLECTION_RECEIVED_MESSAGE = json.dumps({'message': 'Collection received!'})
    COLLECTION_ROUTE = 'collect/'
    COLLECTION_STARTED_STATUS = 'Collection started.'
    COLLECTION_STATUS_KEY = 'status'
    COLLECTION_SUCCESS_MESSAGE = json.dumps({'message': 'Collection Successful!'})
    COLLECTION_SUCCESS_STATUS = 'Collection successful!'
    COLLECTION_WITH_ERRORS_STATUS = 'Collection succeeded with {} errors.'
    COLLECTION_WITH_ERRORS_MESSAGE = json.dumps({'message': 'Collection succeeded with errors.'})
    COLLECTION_MANAGER_TABLE = 'collection_manager'
    COLON = ':'
    COMMA = ','
    COMPLETED = 'completed'
    COMPONENT = 'component'
    CONTENT_TYPE_HEADER = 'Content-Type'
    CONTENT_TYPE_JSON = 'application/json'
    CREATED_SUCCESS_CODE = 201
    MONITOR_ACCOUNT_ID = '12345678'
    DATA_MONITORING_URL_DEFAULT = f'https://insights-collector.newrelic.com/v1/accounts/{MONITOR_ACCOUNT_ID}/events'
    DATA_MONITORING_URL_ENV = 'DATA_MONITORING_URL'
    DATE_FORMAT_KEY = '%d/%m/%y'
    DB_ID_KEY = '_id'
    DEBUG_ROUTE = 'debug/'
    DEFAULT_AWS_ACCOUNT = 'default_aws'
    DEFAULT_DB_NAME = "service_db"
    SUMO_KEY = 'abcd'
    DEFAULT_LOG_URL = f'https://endpoint1.collection.us2.sumologic.com/receiver/v1/http/{SUMO_KEY}'
    DEFAULT_PAGE_OFFSET = 0
    DEFAULT_PAGE_SIZE = 25
    DEFAULT_QUEUE_URL = 'localhost'
    DEFAULT_DYNAMO_THROUGHPUT = {'ReadCapacityUnits': 5, 'WriteCapacityUnits': 5, }
    DELAY_SECONDS_KEY = 'DelaySeconds'
    DELETE_REQUEST = 'DELETE'
    DELETE_REQUEST_MESSAGE = json.dumps({'message': 'Delete Successful!'})
    DEV = 'dev'
    DEV_TOKEN = 'reliabilly.developer_keys'
    DIRECT_FEED = 'direct_feed'
    DEFAULT_AWS_CONNECT_TIMEOUT = 1
    DEFAULT_AWS_MAX_RETRY_COUNT = 0
    AWS_MAX_RETRY_ATTEMPTS = 'max_attempts'
    DYNAMO_RESOURCE_KEY = 'dynamodb'
    DYNAMO_RESPONSE_METADATA = 'ResponseMetadata'
    DYNAMO_SUCCESS_STATUS = 'HTTPStatusCode'
    DYNAMO_ITEMS_KEY = 'Items'
    DYNAMO_ITEM_KEY = 'Item'
    DYNAMO_COUNT_KEY = 'Count'
    DYNAMO_PORT_ENV = 'DYNAMO_PORT'
    DYNAMO_PORT_DEFAULT = 8888
    DYNAMO_URL_ENV = 'DYNAMO_URL'
    DYNAMO_URL_DEFAULT = 'http://localhost'
    DYNAMO_TAGS = [{'Key': 'owner', 'Value': 'jblouse@corpseware.com'},
                   {'Key': 'product', 'Value': 'reliabilly'}]
    EMPTY = ''
    ENDPOINT_URL = 'endpoint_url'
    EQUAL = '='
    ERROR = 'Error'
    ERROR_RATE = 'error_rate'
    FORWARD_SLASH = '/'
    FROM = 'from'
    FROM_DATE_PARAMETER = 'from'
    GET_REQUEST = 'GET'
    GITHUB_ORG = 'corpseware'
    GITHUB_REPO = 'reliabilly'
    HOST = 'host'
    HREF = 'href'
    HYPHEN = '-'
    HEALTHCHECK_ENDPOINT = 'ping'
    ID = 'id'  # pylint: disable=invalid-name
    IGNORE_PRODUCT_LIST = ['api', 'mobile']
    JENKINS = 'Jenkins'
    KMS_KEYS = {'us-west-2': 'arn:aws:kms:us-west-2:{AWS_ACCOUNT}:key/{KMS_GUID}',
                'us-east-1': 'arn:aws:kms:us-east-1:{AWS_ACCOUNT}:key/{KMS_GUID}'}
    LEVEL = 'level'
    LINK = 'link'
    LOCAL = 'local'
    LOCAL_API_NR_ENV = 'LOCAL_NR_API'
    LOCAL_API_SUMO_ENV = 'LOCAL_SUMO_API'
    MAX_PAGE_SIZE = 1000
    MESSAGE_ID_KEY = 'MessageId'
    MESSAGE = 'message'
    MESSAGE_RETENTION_PERIOD_SEC = '86400'
    MESSAGE_RETENTION_PERIOD_KEY = 'MessageRetentionPeriod'
    MESSAGES_KEY = 'Messages'
    MONGO_DB_NAME_ENV = 'MONGO_DB_NAME'
    MONGO_PORT_DEFAULT = 27017
    MONITOR_CALLS_SUFFIX = '_calls'
    MONITOR_ELAPSED_SUFFIX = '_elapsed'
    MONITOR_ERROR_KEY = '_err'
    MONITOR_LARGE_KEY = '_large'
    MONITOR_PREFIX = 'reliabilly.'
    MONITOR_INCR_DEFAULT = 1
    NAME = 'name'
    NEWRELIC = 'newrelic'
    NO_COLLECTION_FOUND_MESSAGE = json.dumps({'message': 'Collection not running!'})
    NONE = 'none'
    NOTIFY_CHANNEL_NAME_ENV = 'NOTIFY_CHANNEL_NAME'
    NOTIFY_MESSAGE = 'The build is broken on PR branch: {branch} at Stage: {stage} \n\nResult Output: {result}'
    OKTA_ACTIVE_STATUS = 'ACTIVE'
    OKTA_SESSION_URL = 'https://okta.com/sessions/{session_token}'
    OPTIONS_REQUEST = 'OPTIONS'
    PAGE_OFFSET = 'offset'
    PAGE_SIZE = 'limit'
    PORT = 'port'
    POST_REQUEST = 'POST'
    PROD = 'prod'
    PROD_BRANCH_NAME = 'master'
    PROD_REGION = 'us-east-1'
    PRODUCT = 'product'
    PRODUCTS = 'products'
    PROXY_URL_DEFAULT = 'http://proxy.com:8080'
    PROXY_URL_ENV = 'PROXY_URL'
    PURGE_REQUEST_MESSAGE = json.dumps({'message': 'Purge Successful!'})
    PUT_REQUEST = 'PUT'
    PYTHON_PACKAGE_REPO = 'https://af.com/artifactory/api/pypi/pypi'
    QUEUE_NAME = 'queue_name'
    LOGGER = 'logger'
    QUERY = 'query'
    QUERY_BUILDER_ROUTE = 'build_query/'
    QUERY_EXECUTOR = 'query_executor'
    QUESTION = '?'
    QUEUE_DELAY_SECONDS = '60'
    QUEUE_PORT = 6379
    QUEUE_RETENTION_PERIOD = '86400'
    QUEUE_URL_ENV = 'QUEUE_URL'
    QUEUE_URL_KEY = 'QueueUrl'
    QUEUE_URLS_KEY = 'QueueUrls'
    RB_FLAG = 'rb'
    READ_ONLY = 'r'
    RECEIVED_SUCCESS_CODE = 202
    RECEIPT_HANDLE_KEY = 'ReceiptHandle'
    RELEASE_SPREADSHEET_URL = 'RELEASE_SPREADSHEET_URL'
    RELEASE_SPREADSHEET_URL_DEFAULT = 'https://docs.google.com/forms/d/e/{FORM_ID}/formResponse'
    REMOTE_LOG_URL_ENV = 'REMOTE_LOG_URL'
    RESOURCE = 'resource'
    RESOURCE_NOT_FOUND_CODE = 404
    RESOURCE_NOT_FOUND_MESSAGE = json.dumps({'message': 'Resource not found'})
    RESOURCE_IN_USE_EXCEPTION = 'ResourceInUseException'
    RESTORE_ROUTE = 'restore/'
    RESULT = 'result'
    RESULTS = 'results'
    RESULTS_ROUTE = 'results/'
    RESULTS_EMPTY_MESSAGE = json.dumps({'message': 'No results found.'})
    ROOT_SERVICE = 'app'
    RUN_ROUTE = 'run/'
    SECURITY = 'security'
    SERVICE = 'service'
    SESSION_URL_ENV = 'SESSION_URL'
    SINGLE_QUOTE = "'"
    SHARED_AUTH_TOKEN = 'shared_auth'
    SLACK_ADD_REACTION_TYPE = 'reactions.add'
    SLACK_API_OK_KEY = 'ok'
    SLACK_API_TOKEN_ID = 'RELIABILLY_SLACK_TOKEN'
    SLACK_EMAIL_KEY = 'email'
    SLACK_EPHEMERAL_MESSAGE_TYPE = 'chat.postEphemeral'
    SLACK_ID_KEY = 'id'
    SLACK_OPENED_PR_TITLE = '{0} has passed linting and testing.'
    SLACK_OPENED_PR_MSG = 'A newly opened PR has just passed Jenkins automated checks. ' \
                          'It\'s now ready to be reviewed by a member of the team.'
    SLACK_MEMBERS_KEY = 'members'
    SLACK_POST_MESSAGE_TYPE = 'chat.postMessage'
    SLACK_PROFILE_KEY = 'profile'
    SLACK_UPDATE_MESSAGE_TYPE = 'chat.update'
    SLACK_USERS_LIST_TYPE = 'users.list'
    SLACK_ALERT_TEAM = 'ALERT_TEAM'
    SLACK_ALERT_USER = 'ALERT_USER'
    SLO = 'slo'
    SLOS = 'slos'
    SOURCE_CONTROL_URL_DEFAULT = "https://github.com/api/v3"
    SOURCE_CONTROL_URL_ENV = "SOURCE_CONTROL_URL"
    SQS_CLIENT = 'sqs'
    RESOURCE_TAGS = {'owner': 'jblosue@corpseware.com', 'product': 'reliabilly'}
    STAGE_ENV = 'STAGE_NAME'
    STARTED = 'created'
    STATS_PORT_DEBUG_DEFAULT = 28125
    STATS_PORT_DEBUG_ENV = 'STATS_PORT_DEBUG'
    STATS_PORT_DEFAULT = 8125
    STATS_PORT_ENV = 'STATS_PORT'
    STATS_URL_DEFAULT = '172.17.0.1'
    STATS_URL_ENV = 'STATS_URL'
    STATUS = 'status'
    SUCCESS_RESPONSE_CODE = 200
    SUMO_CATEGORY_HEADER_KEY = 'X-Sumo-Category'
    SUMO_CATEGORY = 'prod/app/json'
    SUMO_HOST_HEADER = 'https/{account}/ecs/global/{service}/{environment}/{version}/ps'
    SUMO_HOST_HEADER_KEY = 'X-Sumo-Host'
    SUMO_LEVEL_KEY = '@l'
    SUMO_MESSAGE_KEY = '@m'
    SUMO_TIMESTAMP_KEY = '@t'
    SUMO_TIMESTAMP_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
    SUMOLOGIC = 'sumologic'
    SUMOLOGIC_MESSAGE_COUNT = 'messageCount'
    SUMOLOGIC_QUERY_URL = 'https://api.us2.sumologic.com/api/v1/search/jobs'
    TABLE = 'table'
    TASK_MANAGEMENT_URL_DEFAULT = 'https://jira.com'
    TASK_MANAGEMENT_URL_ENV = 'TASK_MANAGEMENT_URL'
    THRESHOLD = 'threshold'
    TIMEZONE = 'timezone'
    TITLE = 'title'
    TO_DATE_PARAMETER = 'to'
    TOTAL = 'TOTAL'
    TOPIARY = 'topiary'
    NEWRELIC_QUERY_URL = 'NEWRELIC_QUERY_URL'
    NEWRELIC_DEFAULT_QUERY_URL = 'https://insights-api.newrelic.com/v1/accounts/{account_id}/query?nrql={query}'
    UAT = 'uat'
    UAT_REGION = 'us-west-2'
    UNAUTHORIZED_CODE = 403
    UNAUTHORIZED_REQUEST_MESSAGE = json.dumps({'message': 'Access Denied. Please try again with a valid auth token.'})
    UNDERSCORE = '_'
    UNPROCESSABLE_ENTITY_CODE = 422
    UNPROCESSABLE_ENTITY_MESSAGE = json.dumps({'message': 'Invalid payload.'})
    USERS_EMAIL_QUERY = f'{PRODUCTION_URL_BASE}/users/?email='
    URL_SEPARATOR = '/'
    UTF8 = 'utf-8'
    VALUE = 'value'
    VERSION_CHECK_TIMEOUT = 60
    VERSION_ENV = 'VERSION'
    VERSION_RESPONSE_TEMPLATE = '{{"service": "{service}", "version": "{version}", "build-number": "{build_number}"}}'
    NEWRELIC_ACCOUNT_IDS = {'acct1': 1234567, 'acct2': 1234568}
    NEWRELIC_API_KEYS = {'acct1': 'reliabilly.newrelic_acct1_api_key', 'acct2': 'reliabilly.newrelic_acct2_api_key'}
    YML_EXTENSION = 'yml'
    ENCRYPTED_EXTENSION = 'encrypted'
    DYNAMO_ID_TABLE_KEY = {'AttributeName': ID, 'KeyType': 'HASH'}
    DYNAMO_ID_TABLE_ATTR = {'AttributeName': ID, 'AttributeType': 'S'}
    COLLECTION_RECEIVED_RESPONSE = RECEIVED_SUCCESS_CODE, COLLECTION_RECEIVED_MESSAGE
    PING_ROUTE = 'ping/'
    PING_RESPONSE = '{"answer": "pong"}'
    COUNT_ROUTE = 'count/'
    HEALTH_ROUTE = 'health/'
    VERSION_ROUTE = 'version/'
    PURGE_ROUTE = 'purge/'
    AUTH = 'auth'
    MANAGER = 'manager'
    DATA_CLIENT = 'data_client'
    COLLECT_DATA_CLIENT = 'collect_data_client'
    SECRETS_MANAGER = 'secrets_manager'
    COLLECTOR = 'collector'
    CONTROLLER = 'controller'
    FACTORY = 'factory'
    BODY_KEY = 'Body'
    HTTP_CLIENT = 'http_client'
    RESULTS_CLIENT = 'results_client'
    COLLECTION_REQUESTS = 'collection_requests'
    COLLECTION_RESULTS = 'collection_results'
    RUNNER_FACTORY = 'runner_factory'
    MODEL = 'model'
    QUEUE_RECEIVER_ID = 'SqsReceiver.run'
    SQS_URL_ENV = 'SQS_URL'
    SQS_PORT_ENV = 'SQS_PORT'
    DEFAULT_SQS_URL = 'http://localhost'
    DEFAULT_SQS_PORT = '9324'
    USE_DYNAMO = 'USE_DYNAMO'
    X_QUERY_KEY_HEADER = 'X-Query-Key'
    X_INSERT_KEY_HEADER = 'X-Insert-Key'
    NR_EMPTY_RESULTS_MESSAGE = 'Empty results in return payload.'
    NR_NO_RESULT_ITEM_MESSAGE = 'Results not in return payload.'
    SUMO_RETRY_EXCEEDED_MESSAGE = 'Retry count exceeded on querying results.'
    SUMO_MISSING_COUNT_MESSAGE = 'Missing message count on query results.'
    SUMO_NO_RESULT_FOUND_MESSAGE = 'Sumologic result not found for query.'
    FACETS = 'facets'
    CORRELATION_ID = 'correlation_id'
    IGNORE = 'ignore'
    TIMEOUT = 'timeout'
    DEFAULT_MESSAGE_WAIT_SECONDS = 5
    SQS_CREATION_ATTRIBUTES = {MESSAGE_RETENTION_PERIOD_KEY: MESSAGE_RETENTION_PERIOD_SEC}
    METRIC_EVENT_TYPE_KEY = 'eventType'
    METRIC_NAME_KEY = 'metric_name'
    METRIC_VALUE_KEY = 'metric_value'
    METRIC_TAG_KEY_PREFIX = 'metric_tag_'
    METRIC_EVENT_ID = 'METRIC_EVENT_ID'
    DEFAULT_METRIC_EVENT_ID = 'dev1'
    NEWRELIC_METRICS_INSERT_SECRET_ID = 'reliabilly.newrelic_acct1_api_insert_key'


def get_service_lists():
    service_list = list()
    lint_list = []
    test_list = list()
    compose_list = [Constants.COMPOSE_FILE_NAME]
    for directory in listdir(Constants.CURRENT_DIR):
        if path.exists(path.join(directory, Constants.DOCKER_FILE)):
            service_list.append(directory)
        if path.exists(path.join(directory, Constants.COMPOSE_FILE_NAME)):
            compose_list.append(path.join(directory, Constants.COMPOSE_FILE_NAME))
        if path.exists(path.join(directory, Constants.PYTHON_MODULE_FILE)):
            lint_list.append(directory)
        if path.exists(path.join(directory, Constants.TESTS_DIR)):
            if not listdir(path.join(directory, Constants.TESTS_DIR)) == []:
                test_list.append(directory)
    return service_list, lint_list, test_list, compose_list


# noinspection PyBroadException
class Settings:
    ENV = environ.get(Constants.ENVIRONMENT, None)
    AWS_ACCOUNT = environ.get(Constants.AWS_ACCOUNT, Constants.DEFAULT_AWS_ACCOUNT)
    AWS_REGION = environ.get(Constants.AWS_REGION, Constants.NONE)
    BUILD_BRANCH = environ.get(Constants.BRANCH_NAME_ENV, None)
    BUILD_NUMBER = environ.get(Constants.BUILD_NUMBER_ENV, Constants.NONE)
    BUILD_STAGE = environ.get(Constants.STAGE_ENV, None)
    COLLECTION_ENABLED = bool(environ.get(Constants.COLLECTION_ENABLED, True))
    COLLECTION_INTERVAL_SECONDS = int(environ.get(
        Constants.COLLECTION_INTERVAL_SECONDS_KEY, Constants.COLLECTION_INTERVAL_DEFAULT))
    DATA_MONITORING_URL = environ.get(Constants.DATA_MONITORING_URL_ENV, Constants.DATA_MONITORING_URL_DEFAULT)
    MONGO_DB_NAME = environ.get(Constants.MONGO_DB_NAME_ENV, Constants.DEFAULT_DB_NAME)
    MONGO_PORT = int(environ.get(Constants.MONGO_PORT_ENV, Constants.MONGO_PORT_DEFAULT))
    MONGO_URL = environ.get(Constants.MONGO_URL_ENV, Constants.MONGO_URL_DEFAULT)
    DV_URL = environ.get(Constants.DV_URL_ENV, Constants.DV_URL_DEFAULT)
    DYNAMO_PORT = int(environ.get(Constants.DYNAMO_PORT_ENV, Constants.DYNAMO_PORT_DEFAULT))
    DYNAMO_URL = environ.get(Constants.DYNAMO_URL_ENV, Constants.DYNAMO_URL_DEFAULT)
    SQS_URL = environ.get(Constants.SQS_URL_ENV, Constants.DEFAULT_SQS_URL)
    SQS_PORT = environ.get(Constants.SQS_PORT_ENV, Constants.DEFAULT_SQS_PORT)
    NEWRELIC_QUERY_URL = environ.get(Constants.NEWRELIC_QUERY_URL, Constants.NEWRELIC_DEFAULT_QUERY_URL)
    NOTIFY_CHANNEL = environ.get(Constants.NOTIFY_CHANNEL_NAME_ENV, None)
    PROXY_URL = environ.get(Constants.PROXY_URL_ENV, Constants.PROXY_URL_DEFAULT)
    QUEUE_URL = environ.get(Constants.QUEUE_URL_ENV, Constants.DEFAULT_QUEUE_URL)
    RELEASE_SPREADSHEET_URL = environ.get(Constants.RELEASE_SPREADSHEET_URL, Constants.RELEASE_SPREADSHEET_URL_DEFAULT)
    REMOTE_LOG_URL = environ.get(Constants.REMOTE_LOG_URL_ENV, None)
    SERVICE_NAME = environ.get(Constants.SERVICE_NAME_ENV, 'empty')
    SESSION_URL = environ.get(Constants.SESSION_URL_ENV, Constants.OKTA_SESSION_URL)
    SLACK_API_TOKEN = environ.get(Constants.SLACK_API_TOKEN_ID, None)
    SOURCE_CONTROL_URL = environ.get(Constants.SOURCE_CONTROL_URL_ENV, Constants.SOURCE_CONTROL_URL_DEFAULT)
    STATS_PORT_DEBUG = int(environ.get(Constants.STATS_PORT_DEBUG_ENV, Constants.STATS_PORT_DEBUG_DEFAULT))
    STATS_PORT = int(environ.get(Constants.STATS_PORT_ENV, Constants.STATS_PORT_DEFAULT))
    STATS_URL = environ.get(Constants.STATS_URL_ENV, Constants.STATS_URL_DEFAULT)
    TASK_MANAGEMENT_URL = environ.get(Constants.TASK_MANAGEMENT_URL_ENV, Constants.TASK_MANAGEMENT_URL_DEFAULT)
    VERSION = environ.get(Constants.VERSION_ENV, Constants.DEV)
    USE_DYNAMO = environ.get(Constants.USE_DYNAMO, None)
    LOCAL_NR_API_KEY = environ.get(Constants.LOCAL_API_NR_ENV, None)
    LOCAL_SUMO_API_KEY = environ.get(Constants.LOCAL_API_SUMO_ENV, None)
    AUTH_TOKEN = environ.get(Constants.AUTH_TOKEN, VERSION)
    RUN_SKIPPED = bool(environ.get(Constants.RUN_SKIPPED_ENV, False))
    METRIC_EVENT_ID = environ.get(Constants.METRIC_EVENT_ID, Constants.DEFAULT_METRIC_EVENT_ID)
    try:
        SERVICES, LINT_LIST, TEST_MODULES, COMPOSE_LIST = get_service_lists()
        LINT_MODULES = Constants.SPACE.join(LINT_LIST)
        COMPOSE_FLAGS = Constants.COMPOSE_FILE_FLAG_SEPARATOR + Constants.COMPOSE_FILE_FLAG_SEPARATOR.join(COMPOSE_LIST)
        LOCAL_MONITORING_API_KEY = str(environ.get('LOCAL_MONITORING_API_KEY', ''))
        MONITORING_API_KEY_NAME = str(environ.get('MONITORING_API_KEY_NAME', ''))
    except Exception:  # pylint: disable=broad-except
        # Keep containers from failing if this code runs as it is only for
        # builds and shouldn't be executing on containers.
        pass


def get_base_url(service_name=Settings.SERVICE_NAME):
    if Settings.ENV == Constants.UAT:
        return Constants.UAT_URL_BASE
    if Settings.ENV == Constants.PROD:
        return Constants.PROD_URL_BASE
    if service_name == Constants.ROOT_SERVICE:
        return Constants.LOCAL_STATIC_URL_BASE.format(service_name)
    if not Settings.ENV and Settings.SERVICE_NAME:
        return Constants.LOCAL_SERVICE_URL_BASE.format(service_name)
    return Constants.LOCAL_URL_BASE


def get_raw_service_url(service_name):
    url_base = get_base_url(service_name)
    if service_name == Constants.ROOT_SERVICE:
        return url_base
    return f'{url_base}{service_name}/'


def get_service_collection_url(service_name):
    base_url = get_raw_service_url(service_name)
    return f'{base_url}{Constants.COLLECTION_ROUTE}'


def get_service_url(service_name, offset=None):
    url_base = get_base_url(service_name)
    service_url = f'{url_base}{service_name}/'
    if offset:
        return f'{service_url}?limit=1000&offset={offset}'
    return service_url + '?limit=1000&offset={offset}'


def get_production_service_url(service_name, offset=None):
    service_url = f'{Constants.PRODUCTION_URL_BASE}/{service_name}/'
    if offset:
        return f'{service_url}?limit=1000&offset={offset}'
    return service_url + '?limit=1000&offset={offset}'


class PortMapper:
    next_port = None

    @classmethod
    def get_next_port(cls):
        if not cls.next_port:
            cls.next_port = Constants.STARTING_PORT
        else:
            cls.next_port += Constants.PORT_INCREMENT
        return cls.next_port


def get_compose_environment():
    for service in Settings.SERVICES:
        environ[f'{service.upper()}{Constants.PORT_ENV_SUFFIX}'] = str(PortMapper.get_next_port())
    environ[Constants.MONGO_PORT_ENV] = str(environ.get(Constants.MONGO_PORT_ENV, Constants.DEFAULT_MONGO_PORT))
    environ[Constants.APP_PORT_ENV] = str(Constants.DEFAULT_APP_PORT)
    environ[Constants.MONGO_URL_ENV] = environ.get(Constants.MONGO_URL_ENV, Constants.MONGO_URL_DEFAULT)
    PortMapper.next_port = Constants.STARTING_PORT
    environ['COMPOSE_CONVERT_WINDOWS_PATHS'] = '1'



from unittest import TestCase, skipUnless
from unittest.mock import MagicMock
from reliabilly.components.collection.collection_manager import CollectionManager
from reliabilly.settings import Constants, Settings


class CollectionManagerTests(TestCase):
    def test_collection_manager(self):
        mock = MagicMock()
        manager = CollectionManager(client=mock)
        new_key = manager.collection_started('dummy')
        self.assertIsNotNone(new_key)
        self.assertTrue(manager.collection_completed(new_key, True))
        self.assertTrue(manager.collection_completed(new_key, False))
        status = manager.get_collection_status(new_key)
        self.assertIsNotNone(status)
        mock.find.return_value = {Constants.COLLECTION_STATUS_KEY: Constants.COLLECTION_STARTED_STATUS}
        self.assertTrue(manager.is_collection_running('dummy'))
        mock.find.return_value = {Constants.DB_ID_KEY: 'dummy',
                                  Constants.COLLECTION_STATUS_KEY: Constants.COLLECTION_FAILED_STATUS}
        self.assertFalse(manager.is_collection_running('dummy'))
        mock.find.return_value = None
        status = manager.get_collection_status(new_key)
        self.assertIsNone(status)
        self.assertFalse(manager.is_collection_running('dummy'))

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_real_manager(self):
        test_collector = 'test_collector'
        manager = CollectionManager()
        manager.purge_prior_collections()
        self.assertIsNone(manager.get_collection_status(test_collector))
        self.assertFalse(manager.is_collection_running(test_collector))
        self.assertTrue(manager.collection_started(test_collector))
        self.assertTrue(manager.is_collection_running(test_collector))
        result = manager.get_collection_status(test_collector)
        self.assertEqual(result[Constants.COLLECTION_STATUS_KEY], Constants.COLLECTION_STARTED_STATUS)
        self.assertTrue(manager.collection_completed(test_collector, True))
        result = manager.get_collection_status(test_collector)
        self.assertEqual(result[Constants.COLLECTION_STATUS_KEY], Constants.COLLECTION_SUCCESS_STATUS)
        self.assertFalse(manager.is_collection_running(test_collector))
        self.assertTrue(manager.collection_started(test_collector))
        self.assertTrue(manager.collection_completed(test_collector))
        self.assertFalse(manager.is_collection_running(test_collector))
        self.assertIsNone(manager.get_collection_status(test_collector))



from invoke import task


@task
def publish(ctx):
    print('Building python package...')
    ctx.run('python3 setup.py sdist')
    print('Pushing package to pypi...')
    ctx.run('twine upload dist/*')



import os
from time import sleep
from invoke import task
from reliabilly.tasks.build import INVALID_SERVICE_MESSAGE
from reliabilly.tasks import docker, data
from reliabilly.components.integration.testing import run_service_integration_tests
from reliabilly.settings import Constants, Settings
from reliabilly.components.web.http_requestor import HttpRequestor

WAIT_TIME_SECONDS = 5
NAMEKO_RUN_CMD = 'nameko run {0}.service'
COMMON_TEST_CMD = 'pytest --ignore=reliabilly/tasks {flags} {service} --cov={service}'
DOCKER_PRINT_STATUS_CMD = 'docker ps --format "table {{.ID}}\t{{.Names}}\t{{.Status}}"'
DEFAULT_INT_TARGET = 'http://localhost:8080'


def test_service(ctx, service_name, times):
    os.environ[Constants.SERVICE_NAME_ENV] = service_name
    flags = ''
    if times:
        flags = '--durations=0'
    ctx.run(COMMON_TEST_CMD.format(service=service_name, flags=flags))


@task(optional=['service'])
def start(ctx, service):
    """ Run the app locally by service. """
    service_list = Settings.SERVICES
    if service not in service_list:
        print(INVALID_SERVICE_MESSAGE.format(service))
        return
    print(f'Setting service name to {service}...')
    os.environ[Constants.SERVICE_NAME_ENV] = service
    ctx.run(NAMEKO_RUN_CMD.format(service))


@task
def test(ctx, service=None, run_skipped=False, times=False):
    """ Run all unit tests """
    if run_skipped:
        os.environ[Constants.RUN_SKIPPED_ENV] = str(run_skipped)
    print(f"Running tests (skipped = {run_skipped})...")
    if service:
        test_service(ctx, service, times)
        return
    for service_test in Settings.TEST_MODULES:
        test_service(ctx, service_test, times)


def wait_for_service_health(ctx, service, sleep_time=10, retries=3):
    call_count = 1
    print(f'Waiting for {service} to be healthy...')
    healthy = docker.health(ctx, service)
    while not healthy:
        sleep(sleep_time)
        call_count += 1
        healthy = docker.health(ctx, service)
        if call_count > retries:
            return


def wait_for_local_service_ping(target, service, sleep_time=10, retries=3):
    call_count = 1
    requestor = HttpRequestor()
    print(f'Waiting for {service} ping to succeed...')
    ping_url = f'{target}/{service}/ping/'
    healthy = requestor.get(ping_url).status_code == Constants.SUCCESS_RESPONSE_CODE
    while not healthy:
        sleep(sleep_time)
        call_count += 1
        healthy = requestor.get(ping_url).status_code == Constants.SUCCESS_RESPONSE_CODE
        if call_count > retries:
            return


# noinspection PyBroadException
@task(optional=['target'])
def int_test(ctx, target=DEFAULT_INT_TARGET):
    """ Runs our integration tests. This assumes a running container stack. """
    print(f'Waiting for services to be ready...')
    sleep(WAIT_TIME_SECONDS)
    for service_name in Settings.SERVICES:
        try:
            if target == DEFAULT_INT_TARGET:
                wait_for_service_health(ctx, service_name)
                wait_for_local_service_ping(target, service_name)
                ctx.run(DOCKER_PRINT_STATUS_CMD)
            run_service_integration_tests(
                target, f'{service_name}/integration/')
        except Exception:  # pylint: disable=broad-except
            print(f'Tests failed for {service_name}')
            if target == DEFAULT_INT_TARGET:
                ctx.run(DOCKER_PRINT_STATUS_CMD)
            exit(1)


@task(optional=['target'], aliases=['int'])
def integration(ctx, target=DEFAULT_INT_TARGET):
    """ Bring up a fresh containerized environment and run the integration tests. """
    if target == DEFAULT_INT_TARGET:
        os.environ[Constants.AUTH_TOKEN] = Constants.DEV
        os.environ[Constants.VERSION_ENV] = Constants.DEV
        docker.down(ctx)
        docker.build(ctx)
        docker.up(ctx)
        data.load_test_data(ctx)
    int_test(ctx, target)



import json
import logging
import os
from unittest import TestCase, skipUnless
from unittest.mock import MagicMock
from reliabilly.settings import Settings, Constants
from reliabilly.components.services.newrelic import NewRelicQueryExecutor, ResultTypes


# noinspection SqlDialectInspection
class NewRelicTests(TestCase):
    def setUp(self):
        logging.disable(logging.CRITICAL)

    def test_execute_newrelic_query(self):
        mock_new_relic_query = MagicMock()
        mock = MagicMock()
        mock.get_secret.return_value = Constants.EMPTY
        new_relic_query_response = self.get_test_data(
            'test_files/new_relic_query_response.json')
        mock_new_relic_query.get.return_value = mock_new_relic_query
        mock_new_relic_query.json.return_value = new_relic_query_response
        runner = NewRelicQueryExecutor(mock, mock_new_relic_query, mock)
        results = runner.execute_single_value_query(mock_new_relic_query)
        self.assertEqual(0.42081991001201285, results.value)

    def test_execute_facet_query(self):
        mock_new_relic_query = MagicMock()
        mock = MagicMock()
        mock.get_secret.return_value = Constants.EMPTY
        new_relic_query_response = self.get_test_data(
            'test_files/new_relic_facet_query_response.json')
        mock_new_relic_query.get.return_value = mock_new_relic_query
        mock_new_relic_query.json.return_value = new_relic_query_response
        runner = NewRelicQueryExecutor(mock, mock_new_relic_query, mock)
        results = runner.execute_faceted_query(mock_new_relic_query)
        self.assertEqual(
            100, results.value[Constants.FACETS][0][Constants.RESULTS][0][Constants.RESULT])
        mock_new_relic_query.ok = False
        results = runner.execute_faceted_query(mock_new_relic_query)
        self.assertEqual(ResultTypes.ERROR, results.result)

    def test_execute_query_failure(self):
        mock_new_relic_query = MagicMock()
        mock = MagicMock()
        new_relic_query_response = self.get_test_data(
            'test_files/new_relic_query_empty_response.json')
        mock_new_relic_query.get.return_value = mock_new_relic_query
        mock_new_relic_query.json.return_value = new_relic_query_response
        runner = NewRelicQueryExecutor(mock, mock_new_relic_query, mock)
        results = runner.execute_single_value_query(mock_new_relic_query)
        self.assertEqual(ResultTypes.UNKNOWN, results.result)

    def test_execute_results_missing(self):
        mock_new_relic_query = MagicMock()
        mock = MagicMock()
        new_relic_query_response = self.get_test_data(
            'test_files/new_relic_query_missing_results.json')
        mock_new_relic_query.json.return_value = new_relic_query_response
        mock_new_relic_query.get.return_value = mock_new_relic_query
        runner = NewRelicQueryExecutor(mock, mock_new_relic_query, mock)
        results = runner.execute_single_value_query(mock_new_relic_query)
        self.assertEqual(ResultTypes.UNKNOWN, results.result)

    def test_execute_results_null(self):
        mock_new_relic_query = MagicMock()
        mock = MagicMock()
        new_relic_query_response = self.get_test_data(
            'test_files/new_relic_query_null_response.json')
        mock_new_relic_query.get.return_value = mock_new_relic_query
        mock_new_relic_query.json.return_value = new_relic_query_response
        runner = NewRelicQueryExecutor(mock, mock_new_relic_query, mock)
        results = runner.execute_single_value_query(mock_new_relic_query)
        self.assertEqual(ResultTypes.SUCCESS, results.result)
        self.assertEqual(None, results.value)

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_execute_live(self):
        runner = NewRelicQueryExecutor(Constants.EMPTY)
        results = runner.execute_single_value_query(Constants.EMPTY)
        self.assertEqual(141.71249771118164, results)

    def test_url_build(self):
        account_id = 12345678
        query = "SELECT average(`get_elapsed.median`) as ms " \
                "FROM reliabilly_statsd WHERE app LIKE 'infrastructure'" \
                "since 1 month ago"
        results = NewRelicQueryExecutor(
            Constants.EMPTY).build_url(account_id, query)
        self.assertEqual("https://insights-api.newrelic.com/v1/accounts/"
                         "12345678/query?nrql=SELECT%20average%28%60"
                         "get_elapsed.median%60%29%20as%20ms%20FROM%20"
                         "reliabilly_statsd%20WHERE%20app%20LIKE%20%27"
                         "infrastructure%27since%201%20month%20ago", results)

    @staticmethod
    def get_test_data(file):
        test_path = os.path.join(
            os.path.dirname(__file__), file)
        with open(test_path) as test_file:
            return json.load(test_file)



from invoke import Collection, task
from reliabilly.tasks import docker, build, data, tests, process, deploy, debug, chat, k8s, publish
from reliabilly.scaffold.generator import scaffold_up_service
from reliabilly.settings import Constants, Settings


ns = Collection()  # pylint: disable=invalid-name
ns.add_collection(Collection.from_module(docker))
ns.add_collection(Collection.from_module(process))
ns.add_collection(Collection.from_module(build))
ns.add_collection(Collection.from_module(deploy))
ns.add_collection(Collection.from_module(data))
ns.add_collection(Collection.from_module(tests))
ns.add_collection(Collection.from_module(debug))
ns.add_collection(Collection.from_module(chat))
ns.add_collection(Collection.from_module(k8s))


def get_collections():
    collections = list(ns.collections)
    collections.sort()
    return collections


@task(default=True, pre=[build.lint, tests.test], aliases=['build'])
def default(_):
    """ Default invoke task which should be run often during development. """
    pass


@task(pre=[build.lint, tests.test, tests.integration])
def rebuild(_):
    """ Run lint, unit tests, purge, build, and integration tests. """
    pass


@task(aliases=['generate', 'gen'])
def scaffold(_, service):
    """ Scaffold up a new service. """
    next_priority = Constants.STARTING_PRIORITY + len(Settings.SERVICES)
    scaffold_up_service(service, next_priority)


@task
def assume(_, token):
    """ Assume a role and print out the access tokens for the credentials"""
    debug.assume(token)


@task(optional=['namespace'], aliases=['list'])
def ls(_):  # pylint: disable=invalid-name
    """ List which tasks exist for each namespace. """
    print('The following tasks exist by namespace...')
    print('-' * 100)
    collections = get_collections()
    for collection in collections:
        task_list = list(ns.collections[collection].tasks)
        task_list.sort()
        printed_tasks = ', '.join(task_list)
        print(f'{collection} = {printed_tasks}')


@task(aliases=['ns'])
def namespace(_):
    """ List which namespaces exist. """
    print('The following namespaces exist...')
    print('-' * 100)
    collections = get_collections()
    print('\n'.join(collections))


# The following tasks are runnable from the top level without needed to prepend namespace.
ns.add_task(ls)
ns.add_task(namespace)
ns.add_task(default)
ns.add_task(rebuild)
ns.add_task(assume)
ns.add_task(tests.test)
ns.add_task(build.lint)
ns.add_task(build.pip)
ns.add_task(docker.up)
ns.add_task(docker.down)
ns.add_task(docker.images)
ns.add_task(docker.sh)
ns.add_task(docker.logs)
ns.add_task(docker.tail)
ns.add_task(docker.ps)
ns.add_task(docker.psa)
ns.add_task(docker.rm)
ns.add_task(docker.rmi)
ns.add_task(docker.stop)
ns.add_task(docker.purge)
ns.add_task(docker.raw)
ns.add_task(docker.clean)
ns.add_task(tests.integration)
ns.add_task(tests.int_test)
ns.add_task(tests.start)
ns.add_task(build.services)
ns.add_task(process.pr)
ns.add_task(scaffold)
ns.add_task(process.check_service_versions)
ns.add_task(deploy.terraform_deploy)
ns.add_task(process.wu)
ns.add_task(process.wu_me)
ns.add_task(process.add_secret)
ns.add_task(process.update_secret)
ns.add_task(process.delete_secret)
ns.add_task(process.update_service_secrets)
ns.add_task(process.encrypt_secrets)
ns.add_task(process.decrypt_secrets)
ns.add_task(chat.send_pipeline_message)
ns.add_task(chat.populate_pull_info)
ns.add_task(publish.publish)



import json
from statsd import StatsClient
from reliabilly.settings import Settings, Constants
from reliabilly.components.tools.secret_manager import SecretManager
from reliabilly.components.web.http_requestor import HttpRequestor


def get_default_stats_client():
    return StatsClient(Settings.STATS_URL, Settings.STATS_PORT)


# noinspection PyBroadException
class StatsWebClient:  # pylint: disable=broad-except
    def __init__(self, **kwargs):
        self.web_client = kwargs.get(Constants.HTTP_CLIENT, HttpRequestor())
        self.secrets_manager = kwargs.get(Constants.SECRETS_MANAGER, SecretManager())

    def add_gauge(self, name, value, tags):
        try:
            self._send_metrics_web_request(name, value, tags)
            return True
        except Exception:
            return False

    def add_timing(self, item, time_in_ms, tags):
        try:
            self._send_metrics_web_request(item, time_in_ms, tags)
            return True
        except Exception:
            return False

    def update_increment(self, name, value, tags):
        try:
            self._send_metrics_web_request(name, value, tags)
            return True
        except Exception:
            return False

    def _send_metrics_web_request(self, name, value, tags):
        api_key = self.get_api_key()
        headers = {Constants.X_INSERT_KEY_HEADER: api_key, Constants.CONTENT_TYPE_HEADER: Constants.CONTENT_TYPE_JSON}
        payload = self.get_web_metrics_payload(name, value, tags)
        self.web_client.post(Settings.DATA_MONITORING_URL, data=payload, headers=headers)

    def get_api_key(self, local_key=Settings.LOCAL_NR_API_KEY):
        if local_key:
            return local_key
        return self.secrets_manager.get_secret(Constants.NEWRELIC_METRICS_INSERT_SECRET_ID)

    @staticmethod
    def get_web_metrics_payload(name, value, tags=None):
        metric_dictionary = {Constants.METRIC_EVENT_TYPE_KEY: Settings.METRIC_EVENT_ID,
                             Constants.METRIC_NAME_KEY: name, Constants.METRIC_VALUE_KEY: value}
        metric_dictionary.update({f'{Constants.METRIC_TAG_KEY_PREFIX}{key}': value for (key, value) in tags.items()})
        return json.dumps(metric_dictionary, default=str)


# noinspection PyBroadException
class StatsLogClient:  # pylint: disable=broad-except
    """ StatsLogClient sends metric events to metrics providers such as StatsD, Newrelic, Datadog. """
    def __init__(self, **kwargs):
        self.client = kwargs.get(Constants.CLIENT, get_default_stats_client())

    def add_gauge(self, name, value, tags=None):
        try:
            if tags:
                self.client.gauge(name, value, rate=1, delta=False, tags=tags)
                return True
            self.client.gauge(name, value, rate=1, delta=False)
            return True
        except Exception:
            return False

    def add_timing(self, item, time_in_ms):
        try:
            self.client.timing(item, time_in_ms)
            return True
        except Exception:
            return False

    def update_increment(self, name, value, tags=None):
        try:
            if tags:
                self.client.incr(name, value, tags=tags)
                return True
            self.client.incr(name, value)
            return True
        except Exception:
            return False


def monitor_collection_status(service):
    """
    Simple wrapper to collector functions that return True or False
    Sends metric data points for collection_success on True, collection_failed on False
    :param func: Function to be wrapped
    :param service: Name of the service to associate with posted data points
    :return: Wrapped function
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            statsd_client = StatsLogClient()
            success_message = 'collection_success'
            failure_message = 'collection_failure'
            tags = {'service': service}
            if result:
                statsd_client.update_increment(success_message, Constants.MONITOR_INCR_DEFAULT, tags=tags)
                return result
            statsd_client.update_increment(failure_message, Constants.MONITOR_INCR_DEFAULT, tags=tags)
            return False
        return wrapper
    return decorator



# pylint: disable=invalid-name
import unittest
from unittest import skipUnless
from unittest.mock import MagicMock
from jira import JIRAError

from reliabilly.components.tools.task_management_client import TaskManagementClient
from reliabilly.settings import Constants, Settings


class TaskManagementClientTests(unittest.TestCase):

    def test_close_ticket(self):
        mock = MagicMock()
        client = TaskManagementClient(mock, mock, mock, mock)
        ticket_number = 'dummy'
        client.close_ticket(ticket_number)
        self.assertTrue(mock.transition_issue.called)

    def test_close_ticket_failure(self):
        mock = MagicMock()
        mock.transition_issue.side_effect = JIRAError
        client = TaskManagementClient(mock, mock, mock, mock)
        ticket_number = 'dummy'
        self.assertFalse(client.close_ticket(ticket_number))

    def test_set_awaiting_review(self):
        mock = MagicMock()
        client = TaskManagementClient(mock, mock, mock, mock)
        ticket_number = 'dummy'
        client.set_awaiting_review(ticket_number)
        self.assertTrue(mock.transition_issue.called)

    def test_set_awaiting_review_failure(self):
        mock = MagicMock()
        mock.transition_issue.side_effect = JIRAError
        client = TaskManagementClient(mock, mock, mock, mock)
        ticket_number = 'dummy'
        self.assertFalse(client.set_awaiting_review(ticket_number))

    def test_comment_issue(self):
        mock = MagicMock()
        client = TaskManagementClient(mock, mock, mock, mock)
        ticket_number = 'dummy'
        client.comment_issue(ticket_number, 'test message')
        self.assertTrue(mock.add_comment.called)

    def test_comment_issue_failure(self):
        mock = MagicMock()
        mock.add_comment.side_effect = JIRAError
        client = TaskManagementClient(mock, mock, mock, mock)
        ticket_number = 'dummy'
        self.assertFalse(client.comment_issue(ticket_number, 'test message'))

    def test_get_jira_summary(self):
        mock = MagicMock()
        mock.fields = mock
        mock.summary = 'a test summary'
        summary = TaskManagementClient.get_task_summary(mock)
        self.assertEqual(summary, 'a test summary')

    def test_get_jira_description(self):
        mock = MagicMock()
        mock.fields = mock
        mock.description = 'a test description'
        description = TaskManagementClient.get_task_description(mock)
        self.assertEqual(description, 'a test description')

    def test_get_issues_by_user(self):
        mock = MagicMock()
        mock.search_issues.return_value = [1, 2]
        client = TaskManagementClient(mock, mock, mock, mock)
        issues = client.get_issues_by_user('test_user')
        self.assertEqual(len(issues), 2)

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_real_get_summary(self):
        client = TaskManagementClient('USERNAME', 'PASSWORD', Constants.TASK_MANAGEMENT_URL_DEFAULT)
        issues = client.get_issues_by_user('russell.tyrone.jones')
        self.assertEqual(len(issues), 1)
        client.comment_issue(issues[0], 'this is a test comment')
        self.assertEqual(client.get_task_summary(issues[0]), 'Put remaining functions in jenkinsfile into inv')
        client.set_awaiting_review('1470')
        client.close_ticket('1470')



#!/usr/bin/env python
from setuptools import setup, find_packages
from settings import SetupSettings

# pragma: no cover
setup(name=SetupSettings.PACKAGE_NAME,
      version=SetupSettings.VERSION,
      python_requires=SetupSettings.PYTHON_REQUIRES,
      install_requires=SetupSettings.REQUIREMENTS,
      description=SetupSettings.DESCRIPTION,
      author=SetupSettings.AUTHOR,
      author_email=SetupSettings.AUTHOR_EMAIL,
      include_package_data=True,
      license=SetupSettings.LICENSE,
      packages=find_packages(exclude=SetupSettings.EXCLUDE_MODULES),)



from invoke import task


DEV_NAMESPACE = 'dev'
UAT_NAMESPACE = 'uat'
PROD_NAMESPACE = 'prod'
ALL_NAMESPACES = '--all-namespaces'
KUBERNETES_DELETE_CMD = 'kubectl delete -f'
KUBERNETES_APPLY_CMD = 'kubectl apply -f'
KUBERNETES_GET_CMD = 'kubectl get all'
KUBERNETES_GET_ALL_CMD = f'{KUBERNETES_GET_CMD} {ALL_NAMESPACES}'
KUBERNETES_GET_PODS_CMD = 'kubectl get pods'
KUBERNETES_GET_PODS_ALL_CMD = f'{KUBERNETES_GET_PODS_CMD} {ALL_NAMESPACES}'
KUBERNETES_GET_SERVICES_CMD = 'kubectl get services'
KUBERNETES_GET_SERVICES_ALL_CMD = f'{KUBERNETES_GET_SERVICES_CMD} {ALL_NAMESPACES}'


@task
def delete(ctx, resource_path, namespace=DEV_NAMESPACE):
    """ Deletes prior kubernetes stack for service. """
    print(f'Deleting local k8s stack for {resource_path}...')
    ctx.run(f'{KUBERNETES_DELETE_CMD} "{resource_path}" -n "{namespace}"')


@task
def apply(ctx, resource_path, namespace=DEV_NAMESPACE):
    """ Applies changes to local k8s stack for service. """
    ctx.run(f'{KUBERNETES_APPLY_CMD} "{resource_path}" -n "{namespace}"')


@task
def use_context(ctx, context):
    """Uses the specified context."""
    ctx.run(f'kubectl config use-context "{context}"')


@task
def get(ctx, namespace=DEV_NAMESPACE):
    """ Get all resources from local kubernetes. """
    ctx.run(f'{KUBERNETES_GET_CMD} -n "{namespace}"')


@task
def get_all(ctx):
    """ Get all resources from local kubernetes for all namespaces. """
    ctx.run(KUBERNETES_GET_ALL_CMD)


@task
def pods(ctx, namespace=DEV_NAMESPACE):
    """ Get all pods from local kubernetes. """
    ctx.run(f'{KUBERNETES_GET_PODS_CMD} -n "{namespace}"')


@task
def pods_all(ctx):
    """ Get all pods from local kubernetes for all namespaces. """
    ctx.run(KUBERNETES_GET_PODS_ALL_CMD)


@task
def services(ctx, namespace=DEV_NAMESPACE):
    """ Get all services from local kubernetes. """
    ctx.run(f'{KUBERNETES_GET_SERVICES_CMD} -n "{namespace}"')


@task
def services_all(ctx):
    """ Get all services from local kubernetes for all namespaces. """
    ctx.run(KUBERNETES_GET_SERVICES_ALL_CMD)



from unittest import TestCase, skipUnless
from unittest.mock import MagicMock
from reliabilly.components.web.http_requestor import HttpRequestor
from reliabilly.settings import Settings, Constants


GOOGLE_URL = 'http://google.com'


class HttpRequestorTests(TestCase):
    def setUp(self):
        mock = MagicMock()
        mock.get.return_value = mock
        mock.request.return_value = Constants.DUMMY
        self.requestor = HttpRequestor(client=mock)

    def test_get(self):
        result = self.requestor.get(Constants.DUMMY, headers=[])
        self.assertEqual(result, Constants.DUMMY)

    def test_post(self):
        result = self.requestor.post(Constants.DUMMY, data=[], headers=[])
        self.assertEqual(result, Constants.DUMMY)

    def test_put(self):
        result = self.requestor.put(Constants.DUMMY, headers=[])
        self.assertEqual(result, Constants.DUMMY)

    def test_delete(self):
        result = self.requestor.delete(Constants.DUMMY, headers=[])
        self.assertEqual(result, Constants.DUMMY)

    def test_get_items(self):
        mock = MagicMock()
        mock.request.return_value = mock
        mock.json.return_value = {Constants.DUMMY: [1], 'total': 50, 'limit': 1}
        requestor = HttpRequestor(client=mock)
        result = requestor.get_items(Constants.DUMMY, True, Constants.DUMMY)
        self.assertIsNotNone(result)
        result = requestor.get_items(Constants.DUMMY, False, Constants.DUMMY)
        self.assertIsNotNone(result)

    def test_get_all_items(self):
        mock = MagicMock()
        mock.request.return_value = mock
        mock.json.return_value = {Constants.DUMMY: [1], 'total': 50, 'limit': 1}
        requestor = HttpRequestor(client=mock)
        result = requestor.get_all_items(Constants.DUMMY)
        self.assertIsNotNone(result)

    def test_save_items(self):
        mock = MagicMock()
        mock.request.return_value = mock
        mock.json.return_value = {Constants.DUMMY: [1], 'total': 50, 'limit': 1}
        requestor = HttpRequestor(client=mock, save_fn=mock)
        result = requestor.save_data_items(Constants.DUMMY)
        self.assertTrue(result)

    def test_perform_web_request(self):
        mock = MagicMock()
        mock.request.return_value = mock
        mock.json.return_value = {Constants.DUMMY: [1], 'total': 50, 'limit': 1}
        requestor = HttpRequestor(client=mock)
        result = requestor.perform_web_request(Constants.DUMMY, Constants.DUMMY)
        self.assertIsNotNone(result)
        requestor = HttpRequestor(client=mock, circuit_breaker=mock)
        result = requestor.perform_web_request(Constants.DUMMY, Constants.DUMMY)
        self.assertIsNotNone(result)

    def test_get_auth_value(self):
        self.assertIsNotNone(HttpRequestor.get_auth_value(Constants.DUMMY, Constants.DUMMY))

    @skipUnless(Settings.RUN_SKIPPED, Constants.RUN_SKIPPED_MSG)
    def test_real_get(self):
        requestor = HttpRequestor()
        result = requestor.get(GOOGLE_URL)
        self.assertEqual(result.status_code, Constants.SUCCESS_RESPONSE_CODE)



from unittest import TestCase

from reliabilly.components.models.base import BaseModel, Field, SchemaTypeMappings


class CustomTestModel(BaseModel):
    test: str
    val: int
    dub: float
    lis: list
    dict: dict


class SchemaTestModel(BaseModel):
    test: Field(str, required=True)
    test_int: Field(int, required=True)
    test_not_required: Field(str, required=False)


class NotSchemaTestModel(BaseModel):
    test: str


class ModelTests(TestCase):
    def test_json_serialization(self):
        model = CustomTestModel()
        self.assertIsNotNone(model)
        self.assertEqual(model.to_json(model), '{}')
        self.assertEqual(type(model.from_json('{}')), BaseModel)
        result = model.from_json('{"test": "jjb", "val": 123}')
        self.assertEqual(type(result), BaseModel)
        self.assertEqual(result.test, 'jjb')
        self.assertEqual(result.val, 123)

    def test_model_validation(self):
        model = SchemaTestModel()
        model.test = 'blerch'
        model.test_int = 10
        self.assertTrue(model.validate())

    def test_model_validation_failure(self):
        model = SchemaTestModel()
        self.assertFalse(model.validate())

    def test_validation_with_schema(self):
        model = SchemaTestModel()
        model.test = 'blerch'
        self.assertTrue(model.validate(
            None, {'test': {'required': True, 'type': 'string'}}))

    def test_schema_mapping(self):
        self.assertEqual(SchemaTypeMappings['str'], "string")
        self.assertEqual(SchemaTypeMappings['int'], "integer")
        self.assertEqual(SchemaTypeMappings['bool'], "boolean")
        self.assertEqual(SchemaTypeMappings['datetime.date'], "date")
        self.assertEqual(SchemaTypeMappings['datetime.datetime'], "datetime")
        self.assertEqual(SchemaTypeMappings['float'], "float")
        self.assertEqual(SchemaTypeMappings['list'], "list")
        self.assertEqual(SchemaTypeMappings['set'], "set")

    def test_field_init(self):
        field = Field(str, required=True)
        self.assertEqual(field.schema, {'required': True})
        self.assertEqual(field.type, str)

    def test_to_schema(self):
        schema = BaseModel.to_schema(SchemaTestModel)
        self.assertEqual(schema, {
            'test': {'required': True, 'type': 'string'},
            'test_int': {'required': True, 'type': 'integer'},
            'test_not_required': {'required': False, 'type': 'string'}
        })

    def test_with_json_payload(self):
        model = SchemaTestModel()
        payload = {"test": "things"}
        self.assertFalse(model.validate(payload))

    def test_with_json_payload_pass(self):
        model = SchemaTestModel()
        payload = {"test": "things", "test_int": 10}
        self.assertTrue(model.validate(payload))

    def test_with_json_payload_empty(self):
        model = BaseModel()
        payload = {}
        self.assertTrue(model.validate(payload))

    def test_to_schema_no_field(self):
        schema = BaseModel.to_schema(NotSchemaTestModel)
        self.assertFalse(schema)

    def test_validation_without_schema(self):
        model = NotSchemaTestModel()
        self.assertTrue(model.validate())



# pylint: disable=no-self-use
from unittest import TestCase
from unittest.mock import MagicMock
from reliabilly.components.data.mongo_client import MongoDataClient


class TestMongoDataClient(TestCase):
    def test_real_constructor(self):
        self.assertIsNotNone(MongoDataClient())

    def test_rename_table(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        data_client.rename_table(None, 'new_name')
        collection_mock.rename.assert_called_with('new_name', dropTarget=True)

    def test_table_exists(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        db_mock.collection_names.return_value = ['table_name']
        collection_mock = MagicMock()
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.table_exists('table_name')
        self.assertTrue(result)

    def test_create_true(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        mock_result = MagicMock()
        mock_result.inserted_id = True
        collection_mock.insert_one.return_value = mock_result
        res = data_client.create(None)
        self.assertTrue(res)

    def test_restore_true(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        mock_result = MagicMock()
        mock_result.inserted_id = True
        collection_mock.insert_many.return_value = mock_result
        res = data_client.restore(None)
        self.assertTrue(res)

    def test_create_false(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        mock_result = MagicMock()
        collection_mock.insert_one.return_value = mock_result
        mock_result.inserted_id = None
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        res = data_client.create(None)
        self.assertFalse(res)

    def test_get(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        collection_mock.find_one.return_value = True
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.get(None)
        self.assertEqual(result, True)

    def test_delete(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        client_mock.deleted_count = 1
        collection_mock.delete_one.return_value = client_mock
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.delete(None)
        self.assertEqual(result, True)

    def test_get_specific(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        collection_mock.find_one.return_value = True
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.get_specific(None, db_mock)
        self.assertEqual(result, True)

    def test_find(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        collection_mock.find_one.return_value = True
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.find(None, db_mock)
        self.assertEqual(result, True)

    def test_update(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        collection_mock.find_one_and_replace.return_value = True
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.update(None, db_mock)
        self.assertEqual(result, True)

    def test_get_all(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        collection_mock.find.return_value = []
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.get_all()
        self.assertEqual(result, [])

    def test_read(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        mock = MagicMock()
        mock.limit.return_value = [{'item': 'item_name'}]
        mock.skip.return_value = mock
        mock.count.return_value = 5
        collection_mock.find.return_value = mock
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        result = data_client.read(2, 10, {'name': 'item'})
        self.assertEqual(result, ([{'item': 'item_name'}], 2, 10, 5))

    def test_delete_db(self):
        client_mock = MagicMock()
        client_mock.drop_database.return_value = True
        data_client = MongoDataClient(client=client_mock)
        data_client.delete_db()
        client_mock.drop_database.assert_called()

    def test_count(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        count_mock = MagicMock()
        count_mock.count.return_value = 10
        collection_mock = MagicMock()
        collection_mock.find.return_value = count_mock
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        count = data_client.count()
        self.assertEqual(count, 10)

    def test_purge(self):
        client_mock = MagicMock()
        db_mock = MagicMock()
        collection_mock = MagicMock()
        collection_mock.drop.return_value = True
        client_mock.__getitem__.return_value = db_mock
        db_mock.__getitem__.return_value = collection_mock
        data_client = MongoDataClient(client=client_mock)
        purge = data_client.purge()
        self.assertEqual(purge, True)

    def test_dependency(self):
        mock = MagicMock()
        data_client = MongoDataClient(client=mock)
        self.assertEqual(data_client, data_client.get_dependency(mock))
